﻿/// <reference path="SpeechJS/SpeechJS.js" />
var recorderReady = false;
var playerReady = false;

// 自定义展现器
function SentenceExercisePresenter(option) {
    LogUtil.debug("SentenceExercisePresenter | enter");

    var _this = this;
    var _textType = option.textType;
    var _isTalk = _textType === textTypeConst.talk;
    var _ttsSentences = undefined;
    var _scores = [];
    var _sentTracks = [];
    var _isDisallowSkip = false;

    var _currentSenIndex = 0;
    var _currentParagraphIndex = 0;

    var isNeedClear;
    var clearSenInPlay;
    var isHighlight;

    initialize();
    function initialize() {
        if (_isTalk) {

            isNeedClear = function (indexInfo) {
                if (indexInfo.ParagraphIndex === _currentParagraphIndex)
                    return false;
                return true;
            }

            clearSenInPlay = clearParagraphInPlay;

            isHighlight = function (indexInfo) {
                if (_isDisallowSkip
						&& indexInfo.ParagraphIndex != _currentParagraphIndex) {
                    return false;
                }
                return true;
            }

        } else {

            isNeedClear = function (indexInfo) {
                if (indexInfo.SentenceIndex === _currentSenIndex)
                    return false;
                return true;
            }

            clearSenInPlay = clearSentenceInPlay;

            isHighlight = function (indexInfo) {
                if (_isDisallowSkip
						&& indexInfo.SentenceIndex != _currentSenIndex) {
                    return false;
                }
                return true;
            }

        }
    }

    this.__proxy__.buildHTML = function (ttsInfo) {
        _ttsSentences = ttsInfo;
        return initHTML(ttsInfo, _textType);
    };

    function initHTML(ttsInfo, textType) {
        switch (textType) {
            case textTypeConst.talk: // 与java中PassageType保持一致
                return buildDialogHTML(ttsInfo);
            default:
                return buildPassageHTML(ttsInfo);
        }
    }

    function buildPassageHTML(ttsInfo) {
        var paragraphs = ttsInfo.Paragraphs;
        LogUtil.debug("buildPassageHTML | enter, paragraphs length:"
				+ paragraphs.length);

        var html = "";

        var globalIndex = 0;
        // 添加段落
        for (var paragraphIndex = 0; paragraphIndex < paragraphs.length; paragraphIndex++) {

            var paragraph = paragraphs[paragraphIndex];
            html += '<div class="' + SpeechJS.Defines.ParagraphClass
					+ ' speech-chapter-paragraph" pIndex="' + paragraphIndex
					+ '">';

            // 添加句子
            for (var senIndex = 0; senIndex < paragraph.Sentences.length; senIndex++) {

                html += '<li><div class="' + cssConst.listimg + '"><img iIdx="'
						+ paragraphIndex + '" src="' + imgConst.gray_point_img
						+ '" /></div>';
                html += '<div class="' + cssConst.listtxt + '">';

                var sen = paragraph.Sentences[senIndex];
                html += '<span class="' + SpeechJS.Defines.SentenceClass
						+ '" sIdx="' + senIndex + '">';

                // 添加单词
                var words = sen.Words;
                for (var wIndex = 0; wIndex < words.length; wIndex++) {
                    var word = words[wIndex];
                    if (word.PY == "") {
                        html += '<span class="' + cssConst.symbol_span
                            + '" wIdx="' + wIndex + '">' + word.PT
                            + ' </span>';
                    }
                    else {
                        html += '<span class="' + SpeechJS.Defines.WordClass
                            + '" wIdx="' + wIndex + '">' + word.PT
                            + ' </span>';
                    }
                }

                html += '</span>';

                html += '</div>';
                html += '</li>';
            }

            html += '</div>';
        }

        // 增加一行空白，使进度条不遮挡文字
        html += '<div class="' + cssConst.listblank + '">' + '&nbsp;'
				+ '</div>';

        LogUtil.debug("buildHTML | leave, html:" + html);

        console.log(html);

        return html;
    }

    // 创建HTML元素
    function buildDialogHTML(ttsInfo) {
        var paragraphs = ttsInfo.Paragraphs;
        LogUtil.debug("buildDialogHTML | enter, paragraphs length:"
				+ paragraphs.length);

        var html = "";

        var globalIndex = 0;
        // 添加段落
        for (var paragraphIndex = 0; paragraphIndex < paragraphs.length; paragraphIndex++) {

            var paragraph = paragraphs[paragraphIndex];
            html += '<div class="' + SpeechJS.Defines.ParagraphClass
					+ ' speech-chapter-paragraph" pIndex="' + paragraphIndex
					+ '">';

            html += '<li><div class="' + cssConst.listimg + '"><img iIdx="'
					+ paragraphIndex + '" src="' + imgConst.gray_point_img
					+ '" /></div>';
            html += '<div class="' + cssConst.listtxt + '">';

            // 添加句子
            for (var senIndex = 0; senIndex < paragraph.Sentences.length; senIndex++) {

                var sen = paragraph.Sentences[senIndex];
                html += '<span class="' + SpeechJS.Defines.SentenceClass
						+ '" sIdx="' + senIndex + '">';

                // 添加单词
                var words = sen.Words;
                for (var wIndex = 0; wIndex < words.length; wIndex++) {
                    var word = words[wIndex];
                    if (word.PY == "" && word.PT.indexOf("</Font>") > -1) {
                        html += '<span wIdx="' + wIndex + '" class="'+cssConst.name_span+'" >'
								+ word.PT.replace("</Font>", "") + ' </span>';
                    } else if (word.PY == "") {
                        html += '<span class="' + cssConst.symbol_span
                            + '" wIdx="' + wIndex + '">' + word.PT
                            + ' </span>';
                    }
                    else {
                        html += '<span class="' + SpeechJS.Defines.WordClass
								+ '" wIdx="' + wIndex + '">' + word.PT
								+ ' </span>';
                    }

                    // html += '<span class="' + SpeechJS.Defines.WordClass +'"
                    // wIdx="' + wIndex + '">' + words[wIndex].PT + ' </span>';
                }

                html += '</span>';
            }

            html += '</div>';
            html += '</li>';

            html += '</div>';
        }

        // 增加一行空白，使进度条不遮挡文字
        html += '<div class="' + cssConst.listblank + '">' + '&nbsp;'
				+ '</div>';

        LogUtil.debug("buildHTML | leave, html:" + html);
        return html;
    }

    /*
    * @Public Method
    */
    this.__proxy__.highlight = function (position) {
        var indexInfo = _this.TTS.getIndexByTime(position);

        if (indexInfo.ParagraphIndex > -1 && indexInfo.SentenceIndex > -1
				&& indexInfo.WordIndex > -1) {

            if (isHighlight(indexInfo) === false) {
                return;
            }

            var sen = _this.DOM.Sentences[indexInfo.SentenceIndex];
            sen.Words[indexInfo.WordIndex].style.color = colorConst.highlight;  

            var itemDivNode = sen.parentNode.parentNode;

            // 不满足清除条件 则不清除
            if (isNeedClear(indexInfo)) {
                clearSenInPlay(getCurrentClearNode(), getSentenceStatus(-1));
                _currentSenIndex = indexInfo.SentenceIndex;
                _currentParagraphIndex = indexInfo.ParagraphIndex;
            }
            if (getItemStatus(itemDivNode) != selectStatus.onPlaying) {
                gildeToCurrentSen();

                // 设置当前句子选中播放状态
                setItemInPlay(itemDivNode);
            }
        }
    }

    function setItemInPlay(itemDivNode, pic) {
        if (pic === undefined)
            pic = selectStatus.onPlaying;

        setItemStatus(itemDivNode.children[0].children[0]/* 正在播放 图片 */, pic);
        setItemSelect(itemDivNode.children[1]/* 背景 */, true);
    }

    function clearItemInPlay(itemDivNode, pic) {
        if (pic === undefined)
            pic = selectStatus.onClear;

        setItemStatus(itemDivNode.children[0].children[0]/* 正在播放 图片 */, pic);
        setItemSelect(itemDivNode.children[1]/* 背景 */, false);
    }

    function clearParagraphInPlay(passageNode, pic) {
        for (var s = 0, sLen = passageNode.Sentences.length; s < sLen; ++s) {
            var sen = passageNode.Sentences[s];
            for (var w = 0, wLen = sen.Words.length; w < wLen; ++w) {
                sen.Words[w].style.color = colorConst.primary;
            }
        }

        var itemDivNode = passageNode.children[0];
        clearItemInPlay(itemDivNode, pic);
    }

    function clearSentenceInPlay(sen, pic) {
        for (var i = 0, len = sen.Words.length; i < len; ++i) {
            sen.Words[i].style.color = colorConst.primary;
        }

        var itemDivNode = sen.parentNode.parentNode;
        clearItemInPlay(itemDivNode, pic);
    }

    // 获得要清除的节点 对话以段落为单位 短文以句子为单位
    function getCurrentClearNode() {
        return _isTalk ? _this.DOM.Paragraphs[_currentParagraphIndex]
				: _this.DOM.Sentences[_currentSenIndex];
    }

    function setItemSelect(list_bg_div, isSelect) {
        list_bg_div.className = isSelect ? cssConst.playing_bg
				: cssConst.listtxt;
    }

    function setItemStatus(list_img, state) {
        var imgSrc;
        switch (state) {
            case selectStatus.onPlaying:
                imgSrc = imgConst.playing_img;
                break;

            case selectStatus.onClear:
                imgSrc = imgConst.gray_point_img;
                break;

            case selectStatus.onRecording:
                imgSrc = imgConst.recording_img;
                break;

            /** *****评测完成后状态***** */ 
            case selectStatus.onEvalVeryGood:
                imgSrc = imgConst.eval_very_good_img;
                break;

            case selectStatus.onEvalNotBad:
                imgSrc = imgConst.eval_not_bad_img;
                break;

            case selectStatus.onEvalBad:
                imgSrc = imgConst.eval_bad_img;
                break;

            case selectStatus.onEvalVeryBad:
                imgSrc = imgConst.eval_very_bad_img;
                break;
                
            case selectStatus.onComplete:
                imgSrc = imgConst.complete_img;
                break;
        }

        list_img.src = imgSrc;
    }

    function getItemStatus(itemDivNode) {
        switch (itemDivNode.children[0].children[0].src) {
            case imgConst.playing_img:
                return selectStatus.onPlaying;

            case imgConst.gray_point_img:
                return selectStatus.onClear;

            case imgConst.recording_img:
                return selectStatus.onRecording;

            case imgConst.eval_very_good_img:
                return selectStatus.onEvalVeryGood;

            case imgConst.eval_not_bad_img:
                return selectStatus.onEvalNotBad;

            case imgConst.eval_bad_img:
                return selectStatus.onEvalBad;

            case imgConst.eval_very_bad_img:
                return selectStatus.onEvalVeryBad;
                               
            case imgConst.complete_img:
            	return selectStatus.onComplete;
        }
    }
    // 在数组中标记句子已读
    function pushTrackSent(sentIndex) {
        // 标记句子已读
        if (sentIndex < _sentTracks.length)
            _sentTracks[sentIndex] = 1;
    }

    this.__proxy__.handleTrackInfo = function (trackInfo) {

        // 句子跟踪
        if (trackInfo.TrackType === 2) {

        }
        // 单词跟踪
        else if (trackInfo.TrackType === 3) {
            var wordPanel = _this.DOM.Words[trackInfo.WordIndex];
            if (wordPanel === undefined)
                return;

            var css;
            switch (trackInfo.ReadStatus) {
                // unread  
                case 0:
                    css = "unread";
                    break;
                // checking  
                case 2:
                    css = "checking";
                    break;
                // "read";  
                case 1:
                    css = "read";
                    break;
                default:
                    css = "unknow";
                    break;
            }

            var sent = _this.DOM.Sentences[wordPanel.SentenceIndex];
            if (sent === undefined)
                return;

            if (css == "read") {
                wordPanel.style.color = colorConst.read;
                // 在数组中标记句子已读
                pushTrackSent(wordPanel.SentenceIndex);
            } else if (css == "checking") {
                wordPanel.style.color = colorConst.checking;
            }else if(css=="unread"){
                wordPanel.style.color = colorConst.read;
                // 在数组中标记句子已读
                pushTrackSent(wordPanel.SentenceIndex);
            }

            var wLen = sent.Words.length;
            if (trackInfo.WordIndex == sent.Words[wLen - 1].GlobalIndex
					&& (wordPanel.SentenceIndex + 1) < _this.DOM.Sentences.length) {
                sent = _this.DOM.Sentences[wordPanel.SentenceIndex + 1];
                recordSentLine({
                    TrackSentIndex: wordPanel.SentenceIndex,
                    SentenceIndex: wordPanel.SentenceIndex + 1,
                    ParagraphIndex: sent.ParagraphIndex
                });
            } else {
                recordSentLine({
                    TrackSentIndex: wordPanel.SentenceIndex,
                    SentenceIndex: wordPanel.SentenceIndex,
                    ParagraphIndex: wordPanel.ParagraphIndex
                });
            }
        }
    };

    this.__proxy__.clearHighlight = function () {
        for (var idx = 0; idx < _this.DOM.Words.length; idx++) {
            _this.DOM.Words[idx].style.color = colorConst.primary;
        }

        clearSenInPlay(getCurrentClearNode(), getSentenceStatus(-1));
        _currentSenIndex = 0;
        _currentParagraphIndex = 0;
    };

    this.__class__.check.call(this);

    function getCurrentSenIndex() {
        return _currentSenIndex;
    }

    function modifyLineStatus(selStatus) {
        if (_currentSenIndex >= 0
				&& _currentSenIndex < _ttsSentences.AllSentences.length) {
            var sen = _this.DOM.Sentences[_currentSenIndex];

            // 设置当前句子选中播放状态
            var itemDivNode = sen.parentNode.parentNode;
            setItemStatus(itemDivNode.children[0].children[0]/* 正在播放 图片 */,
					selStatus);
        }
    }

    function recordSentLine(indexInfo) {
        if (_currentSenIndex === indexInfo.SentenceIndex) {
            return;
        }

        var currentSen = _this.DOM.Sentences[_currentSenIndex];
        clearItemInPlay(currentSen.parentNode.parentNode);

        var sen = _this.DOM.Sentences[indexInfo.SentenceIndex];
        setItemInPlay(sen.parentNode.parentNode, selectStatus.onRecording);

        _currentParagraphIndex = indexInfo.ParagraphIndex;
        _currentSenIndex = indexInfo.SentenceIndex;

        gildeToCurrentSen();
    }

    function clearCurrentSelect() {
        var clearNode = getCurrentClearNode();
        var itemDivNode = _isTalk ? clearNode.children[0]
				: clearNode.parentNode.parentNode;
        setItemSelect(itemDivNode.children[1]/* 背景 */, false);
    }

    function resetSenInPlay() {
        clearSenInPlay(getCurrentClearNode(), getSentenceStatus(-1));
    }

    function readyLearn() {
        _currentSenIndex = 0;
        _currentParagraphIndex = 0;
        _scores = [];
        _sentTracks = [];
        for (var idx = 0; idx < _this.DOM.Sentences.length; idx++) {
            var sen = _this.DOM.Sentences[idx];
            clearSentenceInPlay(sen);
            // 所有句子标记未读
            _sentTracks.push(0);
        }
        firstSenHighlight();
        LogUtil.info("readyLearn...");
    }
    
    function firstSenHighlight(){
        _currentParagraphIndex = 0;
        _currentSenIndex = 0;
        var sen = _this.DOM.Sentences[_currentSenIndex];
        setItemInPlay(sen.parentNode.parentNode, selectStatus.onRecording);
    }
    
    function setSentenceScore(idx, s) {
        _currentSenIndex = idx;
        for (var i = _scores.length; i <= _currentSenIndex; i ++) {
            _scores.push(0);
        }
        _scores[_currentSenIndex] = s;
        LogUtil.info("setSentenceScore idx: " + idx + ", score: " + s);
    }

    function getSentenceScore(idx) {
        if (idx < 0)
            idx = _currentSenIndex;

        if (idx >= 0 && idx < _scores.length)
            return _scores[idx];

        return 0;
    }

    function getSentenceStatus(idx) {
        var s = getSentenceScore(idx);
        if (s == 0 && _scores.length == 0){ 
        	if(_sentTracks.length > 0)
                return _sentTracks[_currentSenIndex] === 1 ? selectStatus.onComplete : selectStatus.onClear;
        	return selectStatus.onClear;
        }      
        else if (s < 60)
            return selectStatus.onEvalVeryBad;
        else if (s < 70)
            return selectStatus.onEvalBad;
        else if (s < 80)
            return selectStatus.onEvalNotBad;
        else
            return selectStatus.onEvalVeryGood;
    }

    function gildeToCurrentSen() {
        var idx = getCurrentSenIndex();
        if (idx < 0 || idx > _this.DOM.Sentences.length)
            return;
        if (idx == 0)
            return;
        // if (idx % constParamDefine.glideMod !== 0) return;

        var sen = _this.DOM.Sentences[idx];
        var itemDivNode = sen.parentNode.parentNode;

        // 滚动
        scrollGlide.glide(itemDivNode, constParamDefine.glideDuration);
    }

    function setDisallowSkip(v) {
        _isDisallowSkip = v;
    }

    function setSentenceIndex(paragraph, sent) {
        clearSenInPlay(getCurrentClearNode(), getSentenceStatus(-1));
        _currentSenIndex = sent;
        _currentParagraphIndex = paragraph;
    }
    // 暴露句子已读情况
    function getSentTracks() {
        return _sentTracks;
    }
    
    //显示句子完成情况
    function showCompleteSent() {
    	if(_scores.length > 0) return;
        for (var i = 0, slength = _sentTracks.length; i < slength; i++) {
            if (_sentTracks[i] == 1){
                var sen = _this.DOM.Sentences[i];
                // 设置当前句子选中播放状态
                var itemDivNode = sen.parentNode.parentNode;
                setItemStatus(itemDivNode.children[0].children[0], selectStatus.onComplete);
            }
        }      
    }
    
    //取消评测时清空该数组，保证页面不会显示句子的完成情况
    function cleanSentTracks(){
    	_sentTracks = [];
    }

    this.setSentenceIndex = setSentenceIndex;
    this.readyLearn = readyLearn;
    this.modifyLineStatus = modifyLineStatus;
    this.recordSentLine = recordSentLine;
    this.setSentenceScore = setSentenceScore;
    this.getSentenceStatus = getSentenceStatus;
    this.gildeToCurrentSen = gildeToCurrentSen;
    this.resetSenInPlay = resetSenInPlay;
    this.clearCurrentSelect = clearCurrentSelect;
    this.setDisallowSkip = setDisallowSkip;
    this.getSentTracks = getSentTracks;
    this.firstSenHighlight = firstSenHighlight;
    this.showCompleteSent = showCompleteSent;
    this.cleanSentTracks = cleanSentTracks;
}
SentenceExercisePresenter.prototype = new SpeechJS.PresenterBase();
SentenceExercisePresenter.prototype.constructor = SentenceExercisePresenter;