package com.oosic.net.connectivity;

import java.util.ArrayList;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiManager;

/**
 * It is the connectivity broadcast reciever.
 * @author CHEN JIAN WEN
 *
 */
class ConnectivityBroadcastReceiver extends BroadcastReceiver {

    /**
     * It is the wifi AP state failed which is hide in the wifi manager class.
     */
    private int WIFI_AP_STATE_FAILED;

    /**
     * It is the wifi AP state enabling which is hided in the wifi manager class.
     */
    private int WIFI_AP_STATE_ENABLING;

    /**
     * It is the wifi AP state enable which is hided in the wifi manager class.
     */
    private int WIFI_AP_STATE_ENABLED;

    /**
     * It is the wifi AP state disabling which is hided in the wifi manager class.
     */
    private int WIFI_AP_STATE_DISABLING;

    /**
     * It is the wifi AP state disabled which is hided in the wifi manager class.
     */
    private int WIFI_AP_STATE_DISABLED;

    /**
     * It is the wifi AP state extra key which is hided in the wifi manager.
     */
    private String EXTRA_WIFI_AP_STATE = new String();

    /**
     * It is the wifi AP state changed action which is hided in the wifi manager.
     */
    private String WIFI_AP_STATE_CHANGED_ACTION = new String();

    /**
     * It is the tether state changed action witch is hided in the connectivity manager.
     */
    private String ACTION_TETHER_STATE_CHANGED = new String();

    /**
     * It is the techer extra witch is hided in the connectivity manager.
     */
    private String EXTRA_ACTIVE_TETHER = new String();

    /**
     * It is the connectivity center.
     */
    private ConnectivityCenter mCC;

    /**
     * It is the wifi connectivity state.
     */
    private NetworkInfo.State mWifiCS = NetworkInfo.State.UNKNOWN;

    /**
     * It is the ethernet connectivity state.
     */
    private NetworkInfo.State mEthernetCS = NetworkInfo.State.UNKNOWN;

    /**
     * It is the bluetooth connectivity state.
     */
    private NetworkInfo.State mBluetoothCS = NetworkInfo.State.UNKNOWN;

    /**
     * It is the mobile connectivity state.
     */
    private NetworkInfo.State mMobileCS = NetworkInfo.State.UNKNOWN;

    /**
     * It is the mobile roaming flag.
     */
    private boolean mMobileRoaming = false;

    /**
     * It is the mobile DUN connectivity state.
     */
    private NetworkInfo.State mMobileDUNCS = NetworkInfo.State.UNKNOWN;

    /**
     * It is the mobile DUN roaming flag.
     */
    private boolean mMobileDUNRoaming = false;

    /**
     * It is the mobile HIPRI connectivity state.
     */
    private NetworkInfo.State mMobileHIPRICS = NetworkInfo.State.UNKNOWN;

    /**
     * It is the mobile HIPRI roaming flag.
     */
    private boolean mMobileHIPRIRoaming = false;

    /**
     * It is the mobile MMS connectivity state.
     */
    private NetworkInfo.State mMobileMMSCS = NetworkInfo.State.UNKNOWN;

    /**
     * It is the mobile MMS roaming flag.
     */
    private boolean mMobileMMSRoaming = false;

    /**
     * It is the mobile SUPL connectivity state.
     */
    private NetworkInfo.State mMobileSUPLCS = NetworkInfo.State.UNKNOWN;

    /**
     * It is the mobile SUPL roaming flag.
     */
    private boolean mMobileSUPLRoaming = false;

    /**
     * It is the wimax connectivity state.
     */
    private NetworkInfo.State mWimaxCS = NetworkInfo.State.UNKNOWN;

    /**
     * It is the wifi AP connectivity state.
     */
    private NetworkInfo.State mWifiAPCS = NetworkInfo.State.UNKNOWN;

    /**
     * It constructs the connectivity broadcast receiver.
     * @param cc It is the connectivity center.
     */
    public ConnectivityBroadcastReceiver(ConnectivityCenter cc) {
        super();
        mCC = cc;
        
        try {
            WIFI_AP_STATE_FAILED =
                WifiManager.class.getField("WIFI_AP_STATE_FAILED").getInt(null);
            WIFI_AP_STATE_ENABLING =
                WifiManager.class.getField("WIFI_AP_STATE_ENABLING").getInt(null);
            WIFI_AP_STATE_ENABLED =
                WifiManager.class.getField("WIFI_AP_STATE_ENABLED").getInt(null);
            WIFI_AP_STATE_DISABLING =
                WifiManager.class.getField("WIFI_AP_STATE_DISABLING").getInt(null);
            WIFI_AP_STATE_DISABLED =
                WifiManager.class.getField("WIFI_AP_STATE_DISABLED").getInt(null);
            EXTRA_WIFI_AP_STATE =
                (String)WifiManager.class.getField("EXTRA_WIFI_AP_STATE").get(null);
            WIFI_AP_STATE_CHANGED_ACTION =
                (String) WifiManager.class.getField("WIFI_AP_STATE_CHANGED_ACTION").get(null);
            ACTION_TETHER_STATE_CHANGED =
                (String) ConnectivityManager.class.getField("ACTION_TETHER_STATE_CHANGED").get(null);
            EXTRA_ACTIVE_TETHER = 
                (String) ConnectivityManager.class.getField("EXTRA_ACTIVE_TETHER").get(null);
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onReceive(Context context, Intent intent) {
        final String action = intent.getAction();
        if (ConnectivityManager.CONNECTIVITY_ACTION.equals(action)) {
            final boolean noConn = 
                intent.getBooleanExtra(ConnectivityManager.EXTRA_NO_CONNECTIVITY, false);

            if (noConn) {
                // No connectivity.
                mCC.onNoConnectivity();
            }
    
            final ConnectivityManager connMgr =
                (ConnectivityManager)mCC.mContext.getSystemService(Context.CONNECTIVITY_SERVICE);
    
            NetworkInfo nwInfos[] = connMgr.getAllNetworkInfo();
            for (NetworkInfo info : nwInfos) {
                if (info == null) continue; 
                final int type = info.getType();
                final NetworkInfo.State state = info.getState();
                if (type == ConnectivityManager.TYPE_WIFI) {
                    // Wifi connectivity changed.
                    if (state != mWifiCS) {
                        mCC.onWifiConnectivityChanged(state);
                        mWifiCS = state;
                    }
                } else if (type == 0x09) {// ConnectivityManager.TYPE_ETHERNET
                    // Ethernet connectivity changed.
                    if (state != mEthernetCS) {
                        mCC.onEthernetConnectivityChanged(state);
                        mEthernetCS = state;
                    }
                } else if (type == 0x07) {// ConnectivityManager.TYPE_BLUETOOTH
                    // BlueTooth connectivity changed.
                    if (state != mBluetoothCS) {
                        mCC.onBluetoothConnectivityChanged(state);
                        mBluetoothCS = state;
                    }
                } else if (type == ConnectivityManager.TYPE_MOBILE) {
                    // Mobile connectivity changed.
                    final boolean isRoaming = info.isRoaming();
                    if ((state != mMobileCS) || (isRoaming != mMobileRoaming)) {
                        mCC.onMobileConnectivityChanged(state, isRoaming);
                        mMobileCS = state;
                        mMobileRoaming = isRoaming;
                    }
                } else if (type == ConnectivityManager.TYPE_MOBILE_DUN) {
                    // Mobile DUN connectivity changed.
                    final boolean isRoaming = info.isRoaming();
                    if ((state != mMobileDUNCS) || (isRoaming != mMobileDUNRoaming)) {
                        mCC.onMobileDUNConnectivityChanged(state, isRoaming);
                        mMobileDUNCS = state;
                        mMobileDUNRoaming = isRoaming;
                    }
                } else if (type == ConnectivityManager.TYPE_MOBILE_HIPRI) {
                    // Mobile HIPRI connectivity changed.
                    final boolean isRoaming = info.isRoaming();
                    if ((state != mMobileHIPRICS) || (isRoaming != mMobileHIPRIRoaming)) {
                        mCC.onMobileHIPRIConnectivityChanged(state, isRoaming);
                        mMobileHIPRICS = state;
                        mMobileHIPRIRoaming = isRoaming;
                    }
                } else if (type == ConnectivityManager.TYPE_MOBILE_MMS) {
                    // Mobile MMS connectivity changed.
                    final boolean isRoaming = info.isRoaming();
                    if ((state != mMobileMMSCS) || (isRoaming != mMobileMMSRoaming)) {
                        mCC.onMobileMMSConnectivityChanged(state, isRoaming);
                        mMobileMMSCS = state;
                        mMobileMMSRoaming = isRoaming;
                    }
                } else if (type == ConnectivityManager.TYPE_MOBILE_SUPL) {
                    // Mobile SUPL connectivity changed.
                    final boolean isRoaming = info.isRoaming();
                    if ((state != mMobileSUPLCS) || (isRoaming != mMobileSUPLRoaming)) {
                        mCC.onMobileSUPLConnectivityChanged(state, isRoaming);
                        mMobileSUPLCS = state;
                        mMobileSUPLRoaming = isRoaming;
                    }
                } else if (type == ConnectivityManager.TYPE_WIMAX) {
                    // Wimax connectivity changed.
                    if (state != mWimaxCS) {
                        mCC.onWimaxConnectivityChanged(state);
                        mWimaxCS = state;
                    }
                }
            }
        } else if (action.equals(WIFI_AP_STATE_CHANGED_ACTION)) {
            onApStateChanged(intent);
            return;
        } else if (action.equals(ACTION_TETHER_STATE_CHANGED)) {
            ArrayList<String> active = intent
                    .getStringArrayListExtra(EXTRA_ACTIVE_TETHER);
            if (active != null) {
                mCC.onTetherStateChanged(active);
            }
        }
    }

    /**
     * It process the wifi AP state changed messages.
     * @param intent It is the intent of the boradcast receiver.
     */
    private void onApStateChanged(Intent intent) {
        int state = intent.getIntExtra(EXTRA_WIFI_AP_STATE, WIFI_AP_STATE_FAILED);

        NetworkInfo.State s;
        if (state ==  WIFI_AP_STATE_ENABLING) {
            s = NetworkInfo.State.CONNECTING;
        } else if (state == WIFI_AP_STATE_ENABLED) {
            s = NetworkInfo.State.CONNECTED;
        } else if (state == WIFI_AP_STATE_DISABLING) {
            s = NetworkInfo.State.DISCONNECTING;
        } else if (state == WIFI_AP_STATE_DISABLED) {
            s = NetworkInfo.State.DISCONNECTED;
        } else {
            s = NetworkInfo.State.UNKNOWN;
        }

        if (s != mWifiAPCS) {
            mCC.onWifiApConnectivityChanged(s);
            mWifiAPCS = s;
        }
    }

}
