package com.oosic.net.connectivity;

import java.util.ArrayList;

import android.net.NetworkInfo;

/**
 * It is the listener of the connectivity center. <BR>
 * @author CHEN JIAN WEN
 *
 */
public interface ConnectivityListener {
    /**
     * It is called when there is no connectivity including no wifi AP.
     */
    void onNoConnectivity();

    /**
     * It is called when the ethernet connectivity changed.
     * @param state It is the network state.
     */
    void onEthernetConnectivityChanged(NetworkInfo.State state);

    /**
     * It is called when the wifi connectivity changed.
     * @param state It is the network state.
     */
    void onWifiConnectivityChanged(NetworkInfo.State state);

    /**
     * It is called when the bluetooth connectivity changed.
     * @param state It is the network state.
     */
    void onBluetoothConnectivityChanged(NetworkInfo.State state);

    /**
     * It is called when the mobile connectivity changed.
     * @param state It is the network state.
     * @param isRoaming It is the flag if the mobile is roaming.
     */
    void onMobileConnectivityChanged(NetworkInfo.State state, boolean isRoaming);

    /**
     * It is called when the mobile DUN connectivity changed.
     * @param state It is the network state.
     * @param isRoaming It is the flag if the mobile is roaming.
     */
    void onMobileDUNConnectivityChanged(NetworkInfo.State state, boolean isRoaming);

    /**
     * It is called when the mobile HIPRI connectivity changed.
     * @param state It is the network state.
     * @param isRoaming It is the flag if the mobile is roaming.
     */
    void onMobileHIPRIConnectivityChanged(NetworkInfo.State state, boolean isRoaming);

    /**
     * It is called when the mobile MMS connectivity changed.
     * @param state It is the network state.
     * @param isRoaming It is the flag if the mobile is roaming.
     */
    void onMobileMMSConnectivityChanged(NetworkInfo.State state, boolean isRoaming);

    /**
     * It is called when the mobile SUPL connectivity changed.
     * @param state It is the network state.
     * @param isRoaming It is the flag if the mobile is roaming.
     */
    void onMobileSUPLConnectivityChanged(NetworkInfo.State state, boolean isRoaming);

    /**
     * It is called when the wimax connectivity changed.
     * @param state It is the network state.
     */
    void onWimaxConnectivityChanged(NetworkInfo.State state);

    /**
     * It is called when the wifi AP connectivity changed.
     * @param state It is the network state.
     */
    void onWifiApConnectivityChanged(NetworkInfo.State state);

    /**
     * It is called when the tether state changed.
     * @param extra It is the extra of active tether.
     */
    void onTetherStateChanged(ArrayList<String> extra);

    /**
     * It is called when the listener is unregistered.
     */
    void onUnregistered();
}
