package com.oozic.apps;

/**
 * It is the runnable that is used in the thread which can be stopped.
 * @author CHEN JIAN WEN
 */
public abstract class StopableRunnable implements Runnable {
    /**
     * It is the flag that marks the process should be stop. Programmer can make use of this
     * flag to make the process stop earlier. (Android deprecats the stop method.)
     */
    protected boolean mStop = false;

    /**
     * It marks the stop flag.
     * <BR>
     * <B>Note</B>, you must check the thread state after you mark the stop flag. Because this
     * method does not actually stop the thread.
     */
    public void markStop() {
        mStop = true;
    }

    /**
     * It unmarks the stop flag. If you only instantiate one object, it gives you a way to
     * unmark the stop flag after your marking it stop.
     */
    public void unmarkStop() {
        mStop = false;
    }
}