package com.oozic.apps;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Calendar;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.app.Dialog;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.PointF;
import android.graphics.RectF;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.view.Window;
import android.view.WindowManager;


/**
 * 
 * Util : this class store the public static method with common use
 * 
 */
public class Util{
    public static final int BUILD_PREVIEW = 1;
    public static final int BUILD_LITE = 2;
    public static final int BUILD_RELEASE = 3;
    public static final int BUILD_AMAZON = 4;
    public static final int BUILD_GOOGLE = 5;

    private static final String FILENAME_PROC_VERSION = "/proc/version";
    private static String readLine(String filename) throws IOException {
        BufferedReader reader = new BufferedReader(new FileReader(filename), 256);
        try {
            return reader.readLine();
        } finally {
            reader.close();
        }
    }

    public static String getFormattedKernelVersion() {
        String procVersionStr;

        try {
            procVersionStr = readLine(FILENAME_PROC_VERSION);

            final String PROC_VERSION_REGEX =
                "\\w+\\s+" + /* ignore: Linux */
                "\\w+\\s+" + /* ignore: version */
                "([^\\s]+)\\s+" + /* group 1: 2.6.22-omap1 */
                "\\(([^\\s@]+(?:@[^\\s.]+)?)[^)]*\\)\\s+" + /* group 2: (xxxxxx@xxxxx.constant) */
                "\\((?:[^(]*\\([^)]*\\))?[^)]*\\)\\s+" + /* ignore: (gcc ..) */
                "([^\\s]+)\\s+" + /* group 3: #26 */
                "(?:PREEMPT\\s+)?" + /* ignore: PREEMPT (optional) */
                "(.+)"; /* group 4: date */

            Pattern p = Pattern.compile(PROC_VERSION_REGEX);
            Matcher m = p.matcher(procVersionStr);

            if (!m.matches()) {
                //Log.e(TAG, "Regex did not match on /proc/version: " + procVersionStr);
                return "Unavailable";
            } else if (m.groupCount() < 4) {
                //Log.e(LOG_TAG, "Regex match on /proc/version only returned " + m.groupCount()
                //        + " groups");
                return "Unavailable";
            } else {
              return (new StringBuilder(m.group(1)).append("\n").append(
              m.group(2)).toString());
//                return (new StringBuilder(m.group(1)).append("\n").append(
//                        m.group(2)).append(" ").append(m.group(3)).append("\n")
//                        .append(m.group(4))).toString();
            }
        } catch (IOException e) {
//            Log.e(LOG_TAG,
//                "IO Exception when getting kernel version for Device Info screen",
//                e);

            return "Unavailable";
        }
    }

    /** Load file content to String */
    public static String loadFileAsString(String filePath) throws java.io.IOException {
        StringBuffer fileData = new StringBuffer(1000);
        BufferedReader reader = new BufferedReader(new FileReader(filePath));
        char[] buf = new char[1024];
        int numRead=0;
        while((numRead=reader.read(buf)) != -1){
            String readData = String.valueOf(buf, 0, numRead);
            fileData.append(readData);
        }
        reader.close();
        return fileData.toString();
    }

    /** Get the STB MacAddress */
    public static String getEthernetMacAddress(){
        try {
            return loadFileAsString("/sys/class/net/eth0/address")
            .toUpperCase().substring(0, 17);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * It gets the Wifi MAC address.
     * @param context It is the context.
     * @return It returns the Wifi MAC addresss.
     */
    public static String getWifiMacAddress(Context context) {
        boolean getDefault = false;
        @SuppressWarnings("rawtypes")
        Class sp = null;
        Method get;
        String wifiInterface = null;
        try {
            sp = Class.forName("android.os.SystemProperties");
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            getDefault = true;
        }

        if (!getDefault) {

            try {
                get = sp.getMethod("get",String.class, String.class);
                wifiInterface = (String)get.invoke(null, "wifi.interface", null);
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
                getDefault = true;
            } catch (IllegalArgumentException e) {
                e.printStackTrace();
                getDefault = true;
            } catch (IllegalAccessException e) {
                e.printStackTrace();
                getDefault = true;
            } catch (InvocationTargetException e) {
                e.printStackTrace();
                getDefault = true;
            }
        }

        if (!getDefault) {
            if ((wifiInterface == null) || (wifiInterface.equals(""))) {
                getDefault = true;
            } else {
                try {
                    return loadFileAsString("/sys/class/net/" + wifiInterface + "/address").toUpperCase().substring(0, 17);
                } catch (IOException e) {
                    e.printStackTrace();
                    return null;
                }
            }
        }

        String ret = null;
        try {
            ret = loadFileAsString("/sys/class/net/wlan0/address").toUpperCase().substring(0, 17);
        } catch (IOException e) {
            e.printStackTrace();
            WifiManager wfm = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
            WifiInfo info = wfm.getConnectionInfo();
            if (null != info) {
                ret = info.getMacAddress();
            } else {
                ret = null;
            }
        }

        return ret;
    }

    /**
     * Get a new path which does not exist.
     * @param path The old path.
     * @return It returns a new path which does not exist.
     */
    public static String getPrivatePath(final String path) {
        String newPath = new String(path);
        File file = new File(newPath);
        int post = 0;
        while (file.exists()) {
            int id = newPath.lastIndexOf('.');

            int start = id;
            String pre;
            CharSequence c;
            if (start >= 1) {
                do {
                    c = newPath.subSequence(start-1, start);

                    if ((c.charAt(0) >= '0') && (c.charAt(0) <= '9')) {
                        start--;
                        if (start == 0) {
                            break;
                        }
                        continue;
                    }
                } while(false);
            }
            post ++;
            if (start > 0) pre = newPath.substring(0, start);
            else pre = new String("");
            pre += post;
            if (id > 0) newPath = pre + newPath.substring(id);
            else newPath = pre;
            file = new File(newPath);
        }
        return newPath;
    }
    /**
     * writeToCache : write bitmap to path as PNG file
     */
    public static void writeToCache(Bitmap bitmap, String path) {
        if (bitmap != null && path != null) {
            try {
                File file = new File(path);
                file.createNewFile();
                final FileOutputStream fos = new FileOutputStream(file);
                final BufferedOutputStream bos = new BufferedOutputStream(fos, 16384);
                bitmap.compress(Bitmap.CompressFormat.PNG, 100, bos);
                bos.flush();
                bos.close();
                fos.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * writeToCache : write bitmap to path as JPG file
     */
    public static void writeToCacheJPEG(Bitmap bitmap, String path) {
        if (bitmap != null && path != null) {
            try {
                File file = new File(path);
                file.createNewFile();
                final FileOutputStream fos = new FileOutputStream(file);
                final BufferedOutputStream bos = new BufferedOutputStream(fos, 16384);
                bitmap.compress(Bitmap.CompressFormat.JPEG, 100, bos);
                bos.flush();
                bos.close();
                fos.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * writeToCache : write bitmap to path as JPG file
     */
    public static void writeToCacheJPEG(Bitmap bitmap, String path, int quality) {
        if (bitmap != null && path != null) {
            try {
                File file = new File(path);
                file.createNewFile();
                final FileOutputStream fos = new FileOutputStream(file);
                final BufferedOutputStream bos = new BufferedOutputStream(fos, 16384);
                bitmap.compress(Bitmap.CompressFormat.JPEG, quality, bos);
                bos.flush();
                bos.close();
                fos.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
    
    public static void deleteFile(String path){
        File file = new File(path);
        if(file.exists())
            file.delete();
    }
    
    /**
     * copy the file from src to dst.
     * 1.if src is directory or is not exist or failure , return -1.
     * 2.if copy is finished correctly ,return 0.
     * @param srcFile
     * @param dstFile
     * @return
     */
    public static int copyFile(File srcFile, File dstFile) {
        int result = 0;
        if (srcFile.isDirectory()) {
            return -1;
        } else {
            if (!srcFile.exists()) {
                return -1;
            } else {
                if (dstFile.exists())
                    dstFile.delete();

                InputStream in = null;
                OutputStream out = null;
                try {
                    in = new FileInputStream(srcFile);
                    out = new FileOutputStream(dstFile);

                    byte[] buf = new byte[1024];
                    int len;
                    while ((len = in.read(buf)) > 0) {
                        out.write(buf, 0, len);
                    }
                } catch (IOException e) {
                    result = -1;
                } finally {
                    try {
                        if (in != null)
                            in.close();
                        if (out != null)
                            out.close();
                    } catch (IOException e) {
                        result = -1;
                    }
                }
            }
        }
        return result;
    }

    /**
     * whether the file(path) is the type of whiteboard file ( *.owb)
     */
    public static boolean isFileWhiteboard(String path) {
        if (path.endsWith(Settings.OWB_FILE_EXTENSION)) {
            return true;
        } else {
            return false;
        }
    }
    public static boolean isFileTheme(String path) {
        if (path.endsWith(Settings.THEME_FILE_EXTENSION)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * get the whiteboard name from path
     */
    public static String getNameFromPath(String path) {
        //if (path.endsWith(WhiteboardGallery.WHITEBOARD_SUFFIX)) {
            return path.substring(path.lastIndexOf("/") + 1,
                    path.length() - Settings.OWB_FILE_EXTENSION.length());
           
//        } else {
//            return null;
//        }
    }

    /**
     * delete path file , if path is directory delete all files and
     * sub-directory
     */
    public static void deleteDirectory(String path) {
        try {
            File file = new File(path);
            File[] files = file.listFiles();
            for (int i = 0; i < files.length; i++) {
                if (files[i].exists())
                    if (files[i].isDirectory())
                        deleteDirectory(files[i].getAbsolutePath());
                    else
                        files[i].delete();
            }
            file.delete();
        } catch (Exception e) {

        }
    }

    /**
     * return the current date string , Format YYYY/MM/DD
     */
    public static String getCurrentDateString() {
        Calendar calendar = Calendar.getInstance();
        String date = calendar.get(Calendar.YEAR) + "/" + (calendar.get(Calendar.MONTH) + 1) + "/"
                + calendar.get(Calendar.DAY_OF_MONTH);
        return date;
    }

    public static int nextPowerOf2(int n) {
        n -= 1;
        n |= n >>> 1;
        n |= n >>> 2;
        n |= n >>> 4;
        n |= n >>> 8;
        n |= n >>> 16;
        return n + 1;
    }

    public static int prevPowerOf2(int n) {
        if (isPowerOf2(n)) {
            return n;
        } else {
            return nextPowerOf2(n) / 2;
        }
    }

    public static boolean isPowerOf2(int n) {
        return (n & -n) == n;
    }

    public static String getExtName(String name) {
        String ext = "";

        int i = name.lastIndexOf(".");
        if (i >= 0)
            ext = name.substring(i + 1, name.length());

        return ext;
    }

    /**
     * create the folder with selected path.
     * @param path
     */
    public static void createLocalDiskPath(String path) {
        File folder = new File(path);
        try {
            if (!folder.exists()) {

                folder.mkdirs();
            }
        } catch (Exception e) {

        }
    }

    /**
     * create the file with selected path.
     * @param path
     */
    public static void createLocalFile(String path) {
        File f = new File(path);
        try {
            if (!f.exists())
                f.createNewFile();
        } catch (Exception e) {
            
        }
    }
    
    /**
     * clear the files of the folder with the path,if the folder is not exist 
     * then create it.
     * @param path
     * @param filter
     */
    public static void cleanupFolder(String path, FileFilter filter){
        File folder = new File(path);
        //Debug("Util","clean up folder:"+path);
        try {
            if (!folder.exists()) {
                folder.mkdirs();
            }else{
                File[] files = folder.listFiles(filter);
                int size = files.length;
                for (int index = 0; index < size; index++) {
                    // when is a file
                    if (files[index].isFile()) {
                        files[index].delete();
                    }
                }
            }
        } catch (Exception e) {

        }  
    }
    public static boolean IsNetworkAvailable(Context mContext) {
        ConnectivityManager conn = (ConnectivityManager) mContext.getSystemService(Context.CONNECTIVITY_SERVICE);

        if (conn == null)
            return false;
        else {
            NetworkInfo[] info = conn.getAllNetworkInfo();
            if (info != null) {
                for (int i = 0; i < info.length; i++) {
                    if (info[i]!=null && info[i].getState() == NetworkInfo.State.CONNECTED)
                        return true;
                }
            }
        }
        return false;
    }

    public static Bitmap getHttpBitmap(String url) throws MalformedURLException {
        Bitmap bmp = null;
        URL burl = null;
        burl = new URL(url);

        try {
            HttpURLConnection conn = (HttpURLConnection) burl.openConnection();
            conn.setConnectTimeout(5000);
            conn.setDoInput(true);
            conn.connect();
            InputStream is = conn.getInputStream();
            bmp = BitmapFactory.decodeStream(is);
            is.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return bmp;

    }

    static public int SaveFile(String filepath, String httppath) throws MalformedURLException {
        int length = 1024;
        URL url = new URL(httppath);

        try {
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setConnectTimeout(10000);
            conn.setDoInput(true);
            conn.connect();
            InputStream is = conn.getInputStream();
            if (is == null)
                return 0;

            File file = new File(filepath);
            FileOutputStream fos = new FileOutputStream(file);
            byte buff[] = new byte[length];
            int mFileByte = 0;

            do {
                mFileByte = mFileByte + length;
                int numread = is.read(buff);
                if (numread <= 0) {
                    break;
                }
                fos.write(buff, 0, numread);

            } while (true);

            fos.close();

        } catch (Exception e) {
            e.printStackTrace();

            return 0;
        }
        return 1;
    }

    public static PointF getRotatedPoint(PointF s, PointF c, double deg) {
        return getRotatedPoint(s.x, s.y, c.x, c.y, (float) Math.toRadians(-deg));
    }

    /**
     * Get the rotate pointer. This method is useful to check if a pointer is in
     * a rotated rectangle.
     * 
     * @param x
     *            The x of original reference frame.
     * @param y
     *            The y of original reference frame.
     * @param cx
     *            The center x of a rectangle which is going to be checked.
     * @param cy
     *            The center y of a rectangle which is going to be checked.
     * @param r
     *            The rotation factor of a rectangle.
     * @return
     */
    public static PointF getRotatedPoint(final float x, final float y, final float cx, final float cy, final float r) {
        PointF p = new PointF(0, 0);

        final float dx = x - cx;
        final float dy = y - cy;
        float radian = (float) Math.atan2(dy, dx);
        radian -= r;
        float len = (float) Math.sqrt(dx * dx + dy * dy);
        p.x = len * (float) Math.cos(radian) + cx;
        p.y = len * (float) Math.sin(radian) + cy;
        return p;
    }
    /**
     * This method help to get the version name.
     * @return It returns the version name of this application.
     */
    public static final String getVersion(Context context) {
        PackageManager manager = context.getPackageManager();
        String packageName = context.getPackageName();
        try {
            PackageInfo info = manager.getPackageInfo(packageName, 0);
            return info.versionName;
        } catch (NameNotFoundException e) {
            return null;
        }
    }

    public static final String getFileTitle(String path) {
        int index = path.lastIndexOf('/');
        return path.substring(index + 1, path.length()); 
    }

    /**
     * rotate the rectangle in the direction of clockwise
     * return the rotated rectangle.
     * @param degree is positive in clockwise
     * @param origin
     * @return
     */
    public static RectF getRotatedRect(double degree, RectF origin) {
        RectF rect = new RectF(origin);
        degree %= 360;
        if (degree == 0) return rect;
        final double radians = Math.toRadians(degree);
        final float centerX = rect.centerX();
        final float centerY = rect.centerY();
        final float w = rect.width();
        final float h = rect.height();
        final double absCosFactor = Math.abs(Math.cos(radians));
        final double absSinFactor = Math.abs(Math.sin(radians));
        final float widthD2 = (float) (w * absCosFactor + h * absSinFactor) / 2;
        final float heightD2 = (float) (w * absSinFactor + h * absCosFactor) / 2;
        RectF rectRotated = new RectF(
                centerX - widthD2,
                centerY - heightD2,
                centerX + widthD2,
                centerY + heightD2);
        return rectRotated;
    }

    /**
     * rotate the rectangle in the direction of counterclockwise
     * return the rotated rectangle.
     * @param degree is positive in clockwise
     * @param origin
     * @return
     */
    public static RectF getOriginRect(double deg, RectF rotated) {
        RectF rect = new RectF();
        deg %= 360;
        if (deg < 0)
            deg = deg + 360;

        double centerX = rotated.centerX();
        double centerY = rotated.centerY();
        double radians = Math.toRadians(deg);

        double width = Math.abs(rotated.width() * Math.abs(Math.cos(radians)) - rotated.height()
                * Math.abs(Math.sin(radians)))
                / Math.abs(2 * Math.cos(radians) * Math.cos(radians) - 1);
        double height = Math.abs(rotated.width() * Math.abs(Math.sin(radians)) - rotated.height()
                * Math.abs(Math.cos(radians)))
                / Math.abs(2 * Math.sin(radians) * Math.sin(radians) - 1);
        rect.left = (float) (centerX - width / 2);
        rect.right = (float) (centerX + width / 2);
        rect.top = (float) (centerY - height / 2);
        rect.bottom = (float) (centerY + height / 2);
        return rect;
    }

    /**
     * It checks if the software version is a share box version.
     * @param context It is the context of application.
     * @return It returns true if the software is designed for share box.
     */
    public static boolean isShareBoxVersion(Context context) {
        int vc;
        try {
            vc = context.getPackageManager().getPackageInfo(context.getPackageName(), 0).versionCode;
        } catch (NameNotFoundException e) {
            vc = 1;
        }
        return (vc % 7) == 0;
    }
    
    public static String convertUrl(String title) {
        title = title.replace("%", "%25");
        title = title.replace(" ", "%20");
        title = title.replace("!", "%21");
        // title = title.replace("\"", "%22");
        title = title.replace("#", "%23");
        title = title.replace("$", "%24");

        title = title.replace("&", "%26");
        title = title.replace(":", "%3a");
        title = title.replace(";", "%3b");
        title = title.replace("[", "%5b");
        title = title.replace("]", "%5d");
        return title;
    }
   
    /**
     * get the file path for image with the workspace and source path
     * @param srcFilePath
     * @param workspace
     * @return
     */
    public static String getCurrentImagePath(String srcFilePath, String workspace) {String srcStr = new String(srcFilePath);
        srcStr = srcStr.replace("/", "_");
        srcStr = srcStr.replace(".", "_");
        srcStr = srcStr.replace(" ", "_");
        srcStr = srcStr.replace(":", "_");
        File file = new File(srcFilePath);
        if (!srcStr.endsWith("_"))
            srcStr = srcStr + "_";
        if (file.exists())
            srcStr = srcStr + file.lastModified() + "_";
        String result = workspace + srcStr
            + Settings.IMAGE_PREFIX + new Date().getTime()
            + getFileSuffix(srcFilePath);
        return result;
    }
    
    /**
     * get the suffix string for the specific path
     * @param filePath
     * @return
     */
    public static String getFileSuffix(String filePath) {
        String suffix = "";
        int start = filePath.lastIndexOf(".");
        if (start != -1)
            suffix = filePath.substring(start);
        return suffix;
    }

    /**
     * It gets the scale factor of the bitmap.
     * @param width It is the bitmap width.
     * @param height It is the bitmap height.
     * @param pixelSize It is the size of each pixel.
     * @return It returns the scale factor of the bitmap.
     */
    public static float getBitmapScaleFactor(int width, int height, int pixelSize) {
        final int total = width * height * pixelSize;
        if (total <= com.oozic.apps.Settings.MAX_BITMAP_LOAD_SIZE) {
            return 1.0f;
        }
        return ((float)com.oozic.apps.Settings.MAX_BITMAP_LOAD_SIZE / (float)total);
    }

    /**
     * It sets a dialog with position on screen.
     * @param dialog It is the dialog.
     * @param x It is the position to set in x axis.
     * @param y It is the position to set in y axis.
     */
    public static void setDialogPosition(Dialog dialog, int x, int y) {
        Window mWindow = dialog.getWindow();
        WindowManager.LayoutParams lp = mWindow.getAttributes();
        lp.x = x;
        lp.y = y;
        lp.flags = WindowManager.LayoutParams.FLAG_DIM_BEHIND;
        lp.dimAmount = 0.0f;
    }
    public static void setDialogPosition(Dialog dialog, int x, int y, int gravity) {
        Window mWindow = dialog.getWindow();
        WindowManager.LayoutParams lp = mWindow.getAttributes();
        lp.x = x;
        lp.y = y;
        lp.gravity = gravity;
        lp.flags = WindowManager.LayoutParams.FLAG_DIM_BEHIND;
        lp.dimAmount = 0.0f;
    }
    /**
     * It calculates the delta RADIAN.
     * @param x It is the touch point value in x axis.
     * @param y It is the touch point value in y axis.
     * @param centerX It is the center point value in x axis.
     * @param centerY It is the center point value in y axis.
     * @param offsetX It is the difference between current touch point value and last touch point
     *  value in x axis.
     * @param offsetY It is the difference between current touch point value and last touch point
     *  value in y axis.
     * @return It returns the delta RADIAN.
     */
    public static double getDeltaRadians(double x, double y, double centerX, double centerY,
            double offsetX, double offsetY) {
        double rLength = Math.sqrt(Math.pow((x - centerX), 2)
                + Math.pow((y - centerY), 2));
        double length = Math.sqrt(Math.pow(offsetX, 2) + Math.pow(offsetY, 2));
        double radians = length / rLength;
        double originRadians = Math.atan((centerY - y) / (x - centerX));
        double rotatedRadians = Math.atan((centerY - (y + offsetY)) / (x + offsetX - centerX));
        if (originRadians < rotatedRadians) {
            radians = -radians;
        }
        return radians;
    }

    /**
     * It converts a short value to a byte array.
     * @param value It is the short value.
     * @return It returns a 2 bytes long byte array.
     */
    public static final byte[] shortToByteArray(short value) {
        return new byte[] { (byte)(value >>> 8), (byte) value};
    }

    /**
     * It converts a integer value to a byte array.
     * @param value It is the integer value.
     * @return It returns a 4 bytes long byte array.
     */
    public static final byte[] intToByteArray(int value) {
        return new byte[] { (byte) (value >>> 24), (byte) (value >>> 16), (byte) (value >>> 8),
                (byte) value };
    }

    /**
     * It converts a long value to a byte array.
     * @param value It is the long value.
     * @return It returns a 8 bytes long byte array.
     */
    public static final byte[] longToByteArray(long value) {
        return new byte[] { (byte) (value >>> 56), (byte) (value >>> 48), (byte) (value >>> 40),
                (byte) (value >>> 32), (byte) (value >>> 24), (byte) (value >>> 16),
                (byte) (value >>> 8), (byte) value };
    }

    /**
     * It converts a byte array to short.
     * <B>Note</B>, the byte array must not be null and its length should longer than 1.
     * @param b It the byte array to convert.
     * @return It returns a short value.
     */
    public static final short byteArrayToShort(byte[] b) {
        return (short)(((b[0] & 0xff) << 8) | (b[1] & 0xff));
    }

    /**
     * It converts a byte array to integer.
     * <B>Note</B>, the byte array must not be null and its length should longer than 3.
     * @param b It the byte array to convert.
     * @return It returns a integer value.
     */
    public static final int byteArrayToInt(byte[] b) {        
        return ((b[0] & 0xff) << 24) | ((b[1] & 0xff) << 16)
                | ((b[2] & 0xff) << 8) | (b[3] & 0xff);
    }

    /**
     * It converts a byte array to long.
     * <B>Note</B>, the byte array must not be null and its length should longer than 7.
     * @param b It the byte array to convert.
     * @return It returns a long value.
     */
    public static final long byteArrayToLong(byte[] b) { 
        int i1 = ((b[0] & 0xff) << 24) | ((b[1] & 0xff) << 16)
                | ((b[2] & 0xff) << 8) | (b[3] & 0xff);
        int i2 = ((b[4] & 0xff) << 24) | ((b[5] & 0xff) << 16)
                | ((b[6] & 0xff) << 8) | (b[7] & 0xff);

        return ((i1 & 0xffffffffL) << 32) | (i2 & 0xffffffffL);
    }
}
