package com.oozic.bitmap;

import java.util.ArrayList;


/**
 * It is the bitmap data set that can be used in the bitmap ASYNCHRONIZED loader.
 * @author CHEN JIAN WEN
 *
 */
@SuppressWarnings("serial")
public class BitmapDataSet extends ArrayList<BitmapData> {
    /**
     * It is the bitmap spy manager.
     */
    private BitmapSpyManager mSpy;

    /**
     * It is the top using index of bitmap data set.
     */
    private int mTUI;

    /**
     * It is the bottom using index of bitmap data set. 
     */
    private int mBUI;

    /**
     * It constructs the bitmap data set.
     * @param spy It is the bitmap spy manager.
     */
    public BitmapDataSet(BitmapSpyManager spy) {
        super();

        mSpy = spy;
        mTUI = mBUI = 0;
    }

    /**
     * It sets the top using index of the bitmap data set.
     * @param index It is the index to set.
     */
    public void setTopUsingIndex(int index) {
        mTUI = index;
    }

    /**
     * It sets the bottom using index of the bitmap data set.
     * @param index
     */
    public void setBottomUsingIndex(int index) {
        mBUI = index;
    }

    /**
     * It helps to release the bitmap data.
     * @param all If the parameter is set true, the bitmap data set will release all the bitmap.
     * And if the parameter is set to false, the bitmap data set will only release the bitmap datas
     * those who are out of range.
     */
    public void release(boolean all) {
        if (mSpy == null) return;

        int i = 0;

        for (BitmapData p : this) {
            if (all || (i < mTUI) || (i > mBUI)) {
                if (p.bmp != null) {
                    if (!p.bmp.isRecycled()){
                        mSpy.recycle(p.bmp);
                        p.bmp = null;
                    }
                }
            }
            i++;
        }

        //mSpy.report(true);
    }
}
