package com.oozic.net;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;


public class ByteBufferPackage extends DataPackage {

    private byte[] mBuffer;

    public ByteBufferPackage() {
        mType = PKG_TYPE_BYTE_BUFFER;
        mBuffer = null;
    }

    ByteBufferPackage(byte[] buffer) {
        mType = PKG_TYPE_BYTE_BUFFER;
        mBuffer = buffer;
    }

    @Override
    public void setByteBuffer(byte[] buffer) {
        mBuffer = buffer;
    }

    @Override
    public byte[] getByteBuffer() {
        return mBuffer;
    }

    protected void write(DataOutputStream dos) throws IOException {
        writePkgHeader(dos);

        if (mBuffer == null) {
            dos.writeInt(0);
        } else {
            dos.writeInt(mBuffer.length);
            dos.write(mBuffer, 0, mBuffer.length);
        }
    }

    protected void read(DataInputStream dis) throws IOException {
        readPkgHeader(dis);

        int length = dis.readInt();
        if (length == 0) {
            mBuffer = null;
        } else {
            if ((mBuffer == null) || (mBuffer.length != length)) {
                mBuffer = new byte[length];
            }

            dis.readFully(mBuffer, 0, length);
        }
    }

    /**
     * It reads the data from source.
     * @param in It is data source.
     */
    public void read(byte[] in) {
        final byte blen[] = new byte[4];
        int i = DataPackage.PKG_HEADER_SIZE;
        System.arraycopy(in, i, blen, 0, 4);
        i += 4;
        final int len = Utils.byteArrayToInt(blen);
        if (len > 0) {
            final byte b[] = new byte[len];
            System.arraycopy(in, i, b, 0, len);
            mBuffer = b;
        }
    }

    /**
     * It write the data to target buffer.
     * @return It returns the data buffer to write.
     */
    public byte[] write() {
        final int len = mBuffer.length;
        final byte blen[] = Utils.intToByteArray(len);
        int i = DataPackage.PKG_HEADER_SIZE;
        final byte out[] = new byte[len + 4 + i];
        if (len > 0) {

            System.arraycopy(blen, 0, out, i, 4);
            i += 4;
            System.arraycopy(mBuffer, 0, out, i, len);
            return out;
        }
        return null;
    }
}