package com.oozic.net;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;


/**
 * It is the motion event package.
 * @author CHEN JIAN WEN
 *
 */
public class MotionEventPackage extends DataPackage {

    /**
     * It is the motion event package data size.
     */
    public static final int MOTION_EVENT_PACKAGE_DATA_SIZE = 12;

    /**
     * It is the motion event action.
     */
    public int mMotionAction;

    /**
     * It is the offset in x axis.
     */
    public int mOffsetX;

    /**
     * It is the offset in y axis.
     */
    public int mOffsetY;

    /**
     * It constructs the motion event package.
     */
    public MotionEventPackage() {
        mType = PKG_TYPE_MOTION_EVENT;
    }

    /**
     * It sets the motion event of the package.
     * @param action It is the motion event action.
     * @param offsetX It is the offset in x axis.
     * @param offsetY It is the offset in y axis.
     */
    public void setMotionEvent(int action, int offsetX, int offsetY) {
        mMotionAction = action;
        mOffsetX = offsetX;
        mOffsetY = offsetY;
    }

    /**
     * It converts the data to buffer.
     * @return It returns the byte array buffer with data
     */
    private byte[] dataToBuffer() {
        final byte[] data = new byte[MOTION_EVENT_PACKAGE_DATA_SIZE];

        int i = 0;
        System.arraycopy(Utils.intToByteArray(mMotionAction), 0, data, i, 4);
        i += 4;


        if (mOffsetX != 0)
            System.arraycopy(Utils.intToByteArray(mOffsetX), 0, data, i, 4);
        i += 4;

        if (mOffsetY != 0)
            System.arraycopy(Utils.intToByteArray(mOffsetY), 0, data, i, 4);
        i += 4;

        return data;
    }

    /**
     * It converts the buffer to value.
     * @param buffer It is the buffer to convert.
     */
    private void bufferToData(byte[] buffer) {
        byte buf[] = new byte[4];
        int i = 0;
        System.arraycopy(buffer, i, buf, 0, 4);
        mMotionAction = Utils.byteArrayToInt(buf);
        i += 4;

        System.arraycopy(buffer, i, buf, 0, 4);
        mOffsetX = Utils.byteArrayToInt(buf);
        i += 4;
        
        System.arraycopy(buffer, i, buf, 0, 4);
        mOffsetY = Utils.byteArrayToInt(buf);
        i += 4;
    }

    /**
     * It writes data to data output stream.
     * @param dos It is the data output stream to write. 
     */
    protected void write(DataOutputStream dos) throws IOException {
        writePkgHeader(dos);

        final byte buffer[] = dataToBuffer();
        dos.writeInt(buffer.length);
        dos.write(buffer, 0, buffer.length);
    }

    /**
     * It reads data from data input stream.
     * @param dos It is the data input stream to read. 
     */
    protected void read(DataInputStream dis) throws IOException {
        readPkgHeader(dis);

        int length = dis.readInt();
        if (length == MOTION_EVENT_PACKAGE_DATA_SIZE) {
            final byte buffer[] = new byte[MOTION_EVENT_PACKAGE_DATA_SIZE];
            dis.readFully(buffer, 0, length);
            bufferToData(buffer);
        }
    }

    /**
     * It reads the data from source.
     * @param in It is data source.
     */
    public void read(byte[] in) {
        final byte blen[] = new byte[4];
        int i = DataPackage.PKG_HEADER_SIZE;
        System.arraycopy(in, i, blen, 0, 4);
        i += 4;
        final int len = Utils.byteArrayToInt(blen);
        if (len > 0) {
            final byte b[] = new byte[len];
            System.arraycopy(in, i, b, 0, len);
            bufferToData(b);
        }
    }

    /**
     * It write the data to target buffer.
     * @return It returns the data buffer to write.
     */
    public byte[] write() {
        final byte buffer[] = dataToBuffer();
        final int len = buffer.length;
        final byte blen[] = Utils.intToByteArray(len);
        int i = DataPackage.PKG_HEADER_SIZE;
        final byte out[] = new byte[len + 4 + i];
        System.arraycopy(blen, 0, out, i, 4);
        i += 4;
        System.arraycopy(buffer, 0, out, i, len);
        return out;
    }
}