package com.oozic.net;

import java.util.ArrayList;

import com.oosic.net.connectivity.ConnectivityListener;
import com.oozic.time.TimeTrigger;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.NetworkInfo;
import android.net.NetworkInfo.State;
import android.net.wifi.WifiManager;

/**
 * It is the connectivity listener. We use this to listen the change of network state.
 * @author CHEN JIAN WEN
 */
class NetConnectivityListener
    extends BroadcastReceiver
    implements ConnectivityListener {

    /**
     * It is the tag of this class.
     */
    @SuppressWarnings("unused")
    private static final String TAG = "NetConnectivityListener";

    /**
     * It is the net manager.
     */
    private NetManager mNetManager;

    /**
     * It is the repeat refresh timer.
     */
    private TimeTrigger mRefreshTT;

    /**
     * It is the repeat refresh count.
     */
    private int mRefreshCount = -1;

    /**
     * It is the refresh time delays.
     */
    private static final int REFRESH_DELAYS[] = {
        500,
        3388
    };

    private State mWifiAPState = State.UNKNOWN;

    /**
     * It is the repeat refresh connection process. Repeat refresh helps to make stable connection.
     */
    private Runnable mRefreshRun = new Runnable() {
        @Override
        public void run() {
            mNetManager.refreshConnections();
            // Make a mechanism to repeat refresh.
            if (mRefreshCount >= 0) {
                if (mRefreshCount < REFRESH_DELAYS.length) {
                    mRefreshTT.start(REFRESH_DELAYS[mRefreshCount], this);
                    mRefreshCount ++;
                } else {
                    mRefreshCount = -1;
                }
            }
        }
    };


    /**
     * It is the process to monitor the network information.
     */
//    private Runnable mTTRun = new Runnable() {
//            @Override
//            public void run() {
//                if (mContext != null) {
//                    boolean refresh = false;
//                    final ConnectivityManager conMgr =
//                        (ConnectivityManager)mContext.getSystemService(Context.CONNECTIVITY_SERVICE);
//                    NetworkInfo nwInfos[] = conMgr.getAllNetworkInfo();
//                    for (int i = 0; i < nwInfos.length; i ++) {
//                        final NetworkInfo info = nwInfos[i];
//                        if (info.getType() == 0x09) {// ConnectivityManager.TYPE_ETHERNET
//                            if (mEthernet == null) {
//                                refresh = true;
//                            } else {
//                                if (mEthernet.getState() == NetworkInfo.State.CONNECTED) {
//                                    if (info.getState() != NetworkInfo.State.CONNECTED) {
//                                        refresh = true;
//                                    }
//                                } else {
//                                    if (info.getState() == NetworkInfo.State.CONNECTED) {
//                                        refresh = true;
//                                    }
//                                }
//                            }
//                            mEthernet = info;
//                            log_i("-------------");
//                            log_i(info.getTypeName());
//                            log_i("State:"+info.getState().toString());
//                            break;
//                        }
//                    }
//                    if (refresh) {
//                        mNetManager.mHandler.removeCallbacks(mRefreshRun);
//                        mRefreshCount = 0;
//                        mNetManager.mHandler.postDelayed(mRefreshRun, REFRESH_DELAY);
//                    }
//                }
//                mTT.start(10000, this);
//            }
//        };

    /**
     * It constructs the net mananger.
     * @param netMgr It is the net manager.
     */
    NetConnectivityListener (NetManager netMgr) {
        super();

        mNetManager = netMgr;
        mRefreshTT = new TimeTrigger();

        // This broadcast receiver is used to notify the Wi-Fi state changed.
        // It helps onet continue work when there is no tether component.
        IntentFilter itf = new IntentFilter();
        itf.addAction(WifiManager.NETWORK_STATE_CHANGED_ACTION);
        itf.addAction(WifiManager.SUPPLICANT_STATE_CHANGED_ACTION);
        mNetManager.mContext.registerReceiver(this, itf);
    }

//    public void onReceive(Context context, Intent intent) {
//        mContext = context;
//        String action = intent.getAction();
//        log_i("Action:" + action);
//        final ConnectivityManager conMgr =
//            (ConnectivityManager)context.getSystemService(Context.CONNECTIVITY_SERVICE);
//
//        if (action.equals(ConnectivityManager.CONNECTIVITY_ACTION)) {
//            handleConnectivity(conMgr, intent);
//            return;
//        }
//
//        try {
//            final String WIFI_AP_STATE_CHANGED_ACTION =
//                (String) WifiManager.class.getField("WIFI_AP_STATE_CHANGED_ACTION").get(null);
//            final String ACTION_TETHER_STATE_CHANGED =
//                (String) ConnectivityManager.class.getField("ACTION_TETHER_STATE_CHANGED").get(null);
//            final String EXTRA_ACTIVE_TETHER = 
//                (String) ConnectivityManager.class.getField("EXTRA_ACTIVE_TETHER").get(null);
//            if (action.equals(WIFI_AP_STATE_CHANGED_ACTION)) {
//                handleApState(intent);
//                return;
//            } else if (action.equals(ACTION_TETHER_STATE_CHANGED)) {
//                ArrayList<String> active = intent
//                        .getStringArrayListExtra(EXTRA_ACTIVE_TETHER);
//                if (DEBUG) {
//                    for (String p : active) {
//                        log_i(p);
//                    }
//                }
//
//                mNetManager.mHandler.removeCallbacks(mRefreshRun);
//                mRefreshCount = 0;
//                mNetManager.mHandler.postDelayed(mRefreshRun, REFRESH_DELAY_SHORT);
//            }
//        } catch (IllegalArgumentException e) {
//            Log.e(TAG, e.toString());
//        } catch (IllegalAccessException e) {
//            Log.e(TAG, e.toString());
//        } catch (SecurityException e) {
//            Log.e(TAG, e.toString());
//        } catch (NoSuchFieldException e) {
//            Log.e(TAG, e.toString());
//        } catch (Exception e) {
//            Log.e(TAG, e.toString());
//        }
//    }

//    private void handleConnectivity(ConnectivityManager conMgr, Intent intent) {
//        boolean refresh = false;
//        NetworkInfo nwInfos[] = conMgr.getAllNetworkInfo();
//        for (int i = 0; i < nwInfos.length; i ++) {
//            final NetworkInfo info = nwInfos[i];
//            if (info.getType() == ConnectivityManager.TYPE_WIFI) {
//                if (mWifi == null) {
//                    refresh = true;
//                } else {
//                    if (mWifi.getState() == NetworkInfo.State.CONNECTED) {
//                        if (info.getState() != NetworkInfo.State.CONNECTED) {
//                            refresh = true;
//                        }
//                    } else {
//                        if (info.getState() == NetworkInfo.State.CONNECTED) {
//                            refresh = true;
//                        }
//                    }
//                }
//                mWifi = info;
//            } else if (info.getType() == 0x09) {// ConnectivityManager.TYPE_ETHERNET
//                if (mEthernet == null) {
//                    refresh = true;
//                } else {
//                    if (mEthernet.getState() == NetworkInfo.State.CONNECTED) {
//                        if (info.getState() != NetworkInfo.State.CONNECTED) {
//                            refresh = true;
//                        }
//                    } else {
//                        if (info.getState() == NetworkInfo.State.CONNECTED) {
//                            refresh = true;
//                        }
//                    }
//                }
//                mEthernet = info;
//            }
//            log_i("-------------");
//            log_i(info.getTypeName());
//            log_i("State:"+info.getState().toString());
//        }
//
//        if (refresh) {
//            mNetManager.mHandler.removeCallbacks(mRefreshRun);
//            mRefreshCount = 0;
//            mNetManager.mHandler.postDelayed(mRefreshRun, REFRESH_DELAY);
//        }
//    }

//    @SuppressWarnings("rawtypes")
//    private void handleApState(Intent intent) {
//        try {
//            final Class ownerClass = WifiManager.class;
//            final String EXTRA_WIFI_AP_STATE =
//                (String)ownerClass.getField("EXTRA_WIFI_AP_STATE").get(null);
//            final int WIFI_AP_STATE_FAILED =
//                ownerClass.getField("WIFI_AP_STATE_FAILED").getInt(null);
//            final int WIFI_AP_STATE_ENABLING =
//                ownerClass.getField("WIFI_AP_STATE_ENABLING").getInt(null);
//            final int WIFI_AP_STATE_ENABLED =
//                ownerClass.getField("WIFI_AP_STATE_ENABLED").getInt(null);
//            final int WIFI_AP_STATE_DISABLING =
//                ownerClass.getField("WIFI_AP_STATE_DISABLING").getInt(null);
//            final int WIFI_AP_STATE_DISABLED =
//                ownerClass.getField("WIFI_AP_STATE_DISABLED").getInt(null);
//
//            int state = intent.getIntExtra(EXTRA_WIFI_AP_STATE, WIFI_AP_STATE_FAILED);
//
//            if (state ==  WIFI_AP_STATE_ENABLING) {
//                log_i("WIFI_AP_STATE_ENABLING");
//            } else if (state == WIFI_AP_STATE_ENABLED) {
//                log_i("WIFI_AP_STATE_ENABLED");
//                mNetManager.mHandler.removeCallbacks(mRefreshRun);
//                mRefreshCount = 0;
//                mNetManager.mHandler.postDelayed(mRefreshRun, REFRESH_DELAY);
//            } else if (state == WIFI_AP_STATE_DISABLING) {
//                log_i("WIFI_AP_STATE_DISABLING");
//            } else if (state == WIFI_AP_STATE_DISABLED) {
//                log_i("WIFI_AP_STATE_DISABLED");
//                mNetManager.mHandler.removeCallbacks(mRefreshRun);
//                mRefreshCount = 0;
//                mNetManager.mHandler.postDelayed(mRefreshRun, REFRESH_DELAY);
//            } else {
//                log_i("WIFI_AP_STATE_XXXX");
//            }
//        } catch (IllegalArgumentException e) {
//            Log.e(TAG, e.toString());
//        } catch (IllegalAccessException e) {
//            Log.e(TAG, e.toString());
//        } catch (SecurityException e) {
//            Log.e(TAG, e.toString());
//        } catch (NoSuchFieldException e) {
//            Log.e(TAG, e.toString());
//        } catch (Exception e) {
//            Log.e(TAG, e.toString());
//        }
//    }

//    /**
//     * It creates an intent filter of the connectivity receiver.
//     * @return It returns an intent filter of the connectivity receiver.
//     */
//    static IntentFilter getIntentFilter() {
//        final IntentFilter filter = new IntentFilter(
//                ConnectivityManager.CONNECTIVITY_ACTION);
//        filter.addAction(WifiManager.WIFI_STATE_CHANGED_ACTION);
//
//        filter.addAction(WifiManager.SUPPLICANT_CONNECTION_CHANGE_ACTION);
//        filter.addAction(WifiManager.NETWORK_STATE_CHANGED_ACTION);
//        filter.addAction(WifiManager.SUPPLICANT_STATE_CHANGED_ACTION);
//        filter.addAction(WifiManager.NETWORK_IDS_CHANGED_ACTION);
//        filter.addAction(WifiManager.RSSI_CHANGED_ACTION);
//        try {
//            final String WIFI_AP_STATE_CHANGED_ACTION =
//                (String) WifiManager.class.getField("WIFI_AP_STATE_CHANGED_ACTION").get(null);
//            final String ACTION_TETHER_STATE_CHANGED =
//                (String) ConnectivityManager.class.getField("ACTION_TETHER_STATE_CHANGED").get(null);
//            filter.addAction(WIFI_AP_STATE_CHANGED_ACTION);
//            filter.addAction(ACTION_TETHER_STATE_CHANGED);
//        } catch (IllegalArgumentException e) {
//            Log.e(TAG, e.toString());
//        } catch (SecurityException e) {
//            Log.e(TAG, e.toString());
//        } catch (IllegalAccessException e) {
//            Log.e(TAG, e.toString());
//        } catch (NoSuchFieldException e) {
//            Log.e(TAG, e.toString());
//        } catch (Exception e) {
//            Log.e(TAG, e.toString());
//        }
//        return filter;
//    }

    /**
     * It start a refresh operation 
     */
    private void refresh() {
        mRefreshCount = 0;
        mRefreshTT.start(1000, mRefreshRun);
    }

    @Override
    public void onNoConnectivity() {
        if (mWifiAPState != State.CONNECTED) {
            // Stop the net manager.
            mNetManager.stop();
        }
    }

    @Override
    public void onEthernetConnectivityChanged(State state) {
        if (state == State.CONNECTED || state == State.DISCONNECTED) {
            refresh();
        }
    }

    @Override
    public void onWifiConnectivityChanged(State state) {
        NetManager.log_i("WifiConnectivityChanged");
        if (state == State.CONNECTED || state == State.DISCONNECTED) {
            refresh();
        }
    }

    @Override
    public void onBluetoothConnectivityChanged(State state) {
    }

    @Override
    public void onMobileConnectivityChanged(State state, boolean isRoaming) {
        if (state == State.CONNECTED || state == State.DISCONNECTED) {
            refresh();
        }
    }

    @Override
    public void onMobileDUNConnectivityChanged(State state, boolean isRoaming) {
        if (state == State.CONNECTED || state == State.DISCONNECTED) {
            refresh();
        }
    }

    @Override
    public void onMobileHIPRIConnectivityChanged(State state, boolean isRoaming) {
        if (state == State.CONNECTED || state == State.DISCONNECTED) {
            refresh();
        }
    }

    @Override
    public void onMobileMMSConnectivityChanged(State state, boolean isRoaming) {
        if (state == State.CONNECTED || state == State.DISCONNECTED) {
            refresh();
        }
    }

    @Override
    public void onMobileSUPLConnectivityChanged(State state, boolean isRoaming) {
        if (state == State.CONNECTED || state == State.DISCONNECTED) {
            refresh();
        }
    }

    @Override
    public void onWimaxConnectivityChanged(State state) {
        if (state == State.CONNECTED || state == State.DISCONNECTED) {
            refresh();
        }
    }

    @Override
    public void onWifiApConnectivityChanged(State state) {
        mWifiAPState = state;
        if (state == State.CONNECTED || state == State.DISCONNECTED) {
            refresh();
        }
    }

    @Override
    public void onUnregistered() {
        mRefreshCount = -1;
        mRefreshTT.clear();
        mNetManager.mContext.unregisterReceiver(this);
    }

    @Override
    public void onTetherStateChanged(ArrayList<String> extra) {
        if (NetManager.DEBUG) {
            for (String p : extra) {
                NetManager.log_i("Tether Extra:" + p);
            }
        }
        refresh();
    }

    @Override
    public void onReceive(Context context, Intent it) {
        final String action = it.getAction();
        if (action.equals(WifiManager.NETWORK_STATE_CHANGED_ACTION)) {
            NetworkInfo ni = (NetworkInfo)it.getParcelableExtra(WifiManager.EXTRA_NETWORK_INFO);
            
            State state = ni.getState();
            NetManager.log_i("Wi-Fi state changed:" + state);
            if ((state == State.CONNECTED) || (state == State.DISCONNECTED)) {
                refresh();
            }
        } else if (action.equals(WifiManager.SUPPLICANT_STATE_CHANGED_ACTION)) {
            NetManager.log_i("Wi-Fi supplicate state changed!");
            refresh();
        }
    }
}