package com.oozic.net;

import java.util.ArrayList;

import com.oozic.apps.Util;

/**
 * It is the container which contains the net data that we suported.
 * @author CHEN JIAN WEN
 * [Not Tested]
 */
public class NetDataContainer extends NetData {

    /**
     * It is the data sets.
     */
    private NetData[] mDatas;

    /**
     * It constructs the net data container.
     * @param id It is the protocol id.
     */
    public NetDataContainer(int id) {
        mID = id;
    }

    /**
     * It constructs the net data container.
     * @param id It is the protocol id.
     * @param datas It is the data it contains.
     */
    public NetDataContainer(int id, NetData ... datas) {
        mID = id;
        mDatas = datas;
    }

    @Override
    public byte[] toBytes() {

        final ArrayList<byte[]> bufferArray = new ArrayList<byte[]>();

        int length = FIXED_HEAD_SIZE;

        bufferArray.add(Util.intToByteArray(mDatas.length * 2));
        length += 8;

        for(int i = 0; i < mDatas.length; i++) {
            byte buffer[];
            final int type = getNetDataType();
            if (type == -1) return null;
            bufferArray.add(Util.intToByteArray(type));
            length += 8;

            buffer = mDatas[i].toBytes();
            if (buffer == null) buffer = new byte[0];
            bufferArray.add(buffer);
            length += 4 + buffer.length;

        }

        final byte mainBuffer[] = new byte[length];
        int i = FIXED_HEAD_SIZE;
        writeHead(mainBuffer, length, bufferArray.size());

        // Set the data size filed
        for (byte[] p : bufferArray) {
            System.arraycopy(Utils.intToByteArray(p.length), 0, mainBuffer, i, 4);
            i += 4;
        }

        // Set the data field
        for (byte[] p : bufferArray) {
            System.arraycopy(p, 0, mainBuffer, i, p.length);
            i += p.length;
        }
        return mainBuffer;
    }

    @Override
    public boolean fromBytes(byte[] bytes) {
        final int lens[] = checkBytes(bytes);

        if (lens == null) return false;
        byte buf[];
        int i = 16 + lens.length * 4;
        int j = 0, j1, n;

        // Get number of data.
        buf = new byte[lens[j]];
        System.arraycopy(bytes, i, buf, 0, lens[j]);
        n = Util.byteArrayToInt(buf);
        mDatas = new NetData[n / 2];
        i += lens[j];
        j ++;
        n += j;

        // Get datas.
        j1 = j;
        for (; j < n ; j++) {
            int type = -1;
            if (lens[j] != 4) {
                return false;
            }
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            type = Util.byteArrayToInt(buf);
            j++;
            i += lens[j];

            if (lens[j] < FIXED_HEAD_SIZE) {
                return false;
            }
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            final NetData d = getNetData(type, buf);
            if (d != null) mDatas[(j - j1) / 2] = d;
            else return false;
            i += lens[j];
        }

        return true;
    }

}
