package com.oozic.net;

import android.util.Log;

import com.oozic.net.NetData;
import com.oozic.net.Utils;

/**
 * It is the data in a file. <BR>
 * If a file data size is too large, generally more than 2MByte, make a manual split.
 * @author CHEN JIAN WEN
 *
 */
public class NetDataFile extends NetData {

    /**
     * It is the class tag.
     */
    private static final String TAG = "NetDataFile";

    /**
     * It is the file data value.
     */
    protected byte[] mData;

    /**
     * It is the index of the file data.
     */
    protected int mIndex;

    /**
     * It constructs the data.
     * @param id It is the protocol id.
     */
    public NetDataFile(int id) {
        this(id, 0, new byte[0]);
    }

    /**
     * It constructs the data.
     * @param id It is the protocol id.
     * @param index It is the index of the data.
     * @param data It is the data buffer.
     */
    public NetDataFile(int id, int index, byte[] data) {
        mID = id;
        if (data == null) {
            mData = new byte[0];
        } else {
            if (data.length > 2097152) {
                Log.w(TAG, "The buffer is too large!");
            }
            mData = data;
        }
        mIndex = index;
    }

    @Override
    public byte[] toBytes() {
        final int length = FIXED_HEAD_SIZE + 8 + 4 + mData.length;

        byte mainBuffer[] = new byte[length];
        int i = FIXED_HEAD_SIZE;

        writeHead(mainBuffer, length, 2);

        final byte[] b4 = Utils.intToByteArray(4);
        // +<<------------------------------------------------------------------------------------+
        // | Set the file data index size.                                                        |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(b4, 0, mainBuffer, i, 4);
        i += 4;
        // +<<------------------------------------------------------------------------------------+
        // | Set the file data size.                                                              |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(mData.length), 0, mainBuffer, i, 4);
        i += 4;

        // +<<------------------------------------------------------------------------------------+
        // | Set the file data index.                                                             |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(mIndex), 0, mainBuffer, i, 4);
        i += 4;
        // +<<------------------------------------------------------------------------------------+
        // | Set the file data.                                                                   |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(mData, 0, mainBuffer, i, mData.length);
        i += mData.length;

        return mainBuffer;
    }

    @Override
    public boolean fromBytes(byte[] bytes) {
        final int lens[] = checkBytes(bytes);

        if (lens == null) return false;
        byte buf[];
        int i = 16 + lens.length * 4;
        int j = 0;

        try {
            // +<<------------------------------------------------------------------------------------+
            // | Get the file data index.                                                             |
            // +--------------------------------------------------------------------------------------+
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            mIndex = Utils.byteArrayToInt(buf);
            i += lens[j];
            j ++;
    
            // +<<------------------------------------------------------------------------------------+
            // | Get the file data.                                                                   |
            // +--------------------------------------------------------------------------------------+
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            mData = buf;
            //i += lens[j];
            //j ++;
        } catch (IndexOutOfBoundsException e) {
            return false;
        }
        return true;
    }

    /**
     * It sets the data buffer.
     * @param data It is the data buffer to set.
     */
    protected void setData(byte[] data) {
        if (data == null) {
            mData = new byte[0];
        } else {
            if (data.length > 2097152) {
                Log.w(TAG, "The buffer is too large!");
            }
            mData = data;
        }
    }

    /**
     * It gets the data index.
     * @return It returns the file data index.
     */
    public int getDataIndex() {
        return mIndex;
    }

    /**
     * It gets the data buffer.
     * @return It returns the file data.
     */
    public byte[] getData() {
        return mData;
    }
}
