package com.oozic.net;

import java.io.File;
import java.io.UnsupportedEncodingException;

/**
 * It defines a net data file head. <BR>
 * Generally, a file head is used as a request to transfer a file.
 * @author CHEN JIAN WEN
 *
 */
public class NetDataFileHead extends NetData {

    /**
     * It is the file path.
     */
    protected String mPath;

    /**
     * It is the file size.
     */
    protected long mSize;

    /**
     * It constructs the file head without initializing its members.
     * @param id It is the protocol id.
     */
    public NetDataFileHead(int id) {
        mID = id;
    }

    /**
     * It constructs the file head.
     * @param id It is the protocol id.
     * @param path It is the file path.
     */
    public NetDataFileHead(int id, String path) {
        mID = id;
        mPath = path;

        final File file = new File(path);
        mSize = file.length();
    }

    @Override
    public byte[] toBytes() {
        byte[] pathBuffer;
        try {
            pathBuffer = mPath.getBytes(STRING_CODING);
        } catch (UnsupportedEncodingException e) {
            pathBuffer = mPath.getBytes();
        } catch (NullPointerException e) {
            pathBuffer = new byte[0];
        }

        final int length = FIXED_HEAD_SIZE + 8 + pathBuffer.length + 8; 

        byte mainBuffer[] = new byte[length];
        int i = ITEM_SIZE_OFFSET;

        writeHead(mainBuffer, length, 2);

        // +<<------------------------------------------------------------------------------------+
        // | Set the size of file path string buffer.                                             |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(pathBuffer.length), 0, mainBuffer, i, 4);
        i += 4;
        // +<<------------------------------------------------------------------------------------+
        // | Set the size of file data size, it's always 8.                                       |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(8), 0, mainBuffer, i, 4);
        i += 4;

        // +<<------------------------------------------------------------------------------------+
        // | Set the data of file path.                                                           |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(pathBuffer, 0, mainBuffer, i, pathBuffer.length);
        i += pathBuffer.length;
        // +<<------------------------------------------------------------------------------------+
        // | Set the data of file size.                                                           |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.longToByteArray(mSize), 0, mainBuffer, i, 8);
        //i += 8;

        return mainBuffer;
    }

    @Override
    public boolean fromBytes(byte[] bytes) {
        final int lens[] = checkBytes(bytes);

        if (lens == null) return false;
        byte buf[];
        int i = FIXED_HEAD_SIZE + lens.length * 4;
        int j = 0;

        try {
            // +<<------------------------------------------------------------------------------------+
            // | Get the file path.                                                                   |
            // +--------------------------------------------------------------------------------------+
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            try {
                mPath = new String(buf, STRING_CODING);
            } catch (UnsupportedEncodingException e) {
                mPath = new String(buf);
            }
            i += lens[j];
            j ++;
    
            // +<<------------------------------------------------------------------------------------+
            // | Get the file size.                                                                   |
            // +--------------------------------------------------------------------------------------+
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            mSize = Utils.byteArrayTolong(buf);
            //i += lens[j];
            //j ++;
        } catch (IndexOutOfBoundsException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    /**
     * It gets the path.
     * @return It returns the path.
     */
    public String getPath() {
        return mPath;
    }

    /**
     * It gets the size of
     * @return It returns the size.
     */
    public long getSize() {
        return mSize;
    }
}
