package com.oozic.net;

import com.oozic.net.NetData;
import com.oozic.net.Utils;

/**
 * It is the data that contains integers.
 * @author CHEN JIAN WEN
 *
 */
public class NetDataIntegers extends NetData {
    /**
     * It is integer value set.
     */
    protected int[] mValues;

    /**
     * It constructs the data.
     */
    public NetDataIntegers(int id) {
        this(id, 0);
    }

    /**
     * It constructs the data.
     * @volume It is the music volume
     */
    public NetDataIntegers(int id, int ...values) {
        mValues = values;
        mID = id;
    }

    @Override
    public byte[] toBytes() {

        final int length =
            24 +
            (4 + 4) * mValues.length;

        final byte[] buf = new byte[length];
        int i = 0;

        // +<<------------------------------------------------------------------------------------+
        // | Set protocol id.                                                                     |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(mID), 0, buf, i, 4);
        i += 4;

        // +<<------------------------------------------------------------------------------------+
        // | Set key.                                                                             |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(mKey), 0, buf, i, 4);
        i += 4;

        // +<<------------------------------------------------------------------------------------+
        // | Set whole data length.                                                               |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(length), 0, buf, i, 4);
        i += 4;

        // +<<------------------------------------------------------------------------------------+
        // | Set data field number.                                                               |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(mValues.length + 1), 0, buf, i, 4);
        i += 4;

        final byte[] b4 = Utils.intToByteArray(4);
        // +<<------------------------------------------------------------------------------------+
        // | Set integer data number length.                                                      |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(b4, 0, buf, i, 4);
        i += 4;
        // +<<------------------------------------------------------------------------------------+
        // | Set integer data length.                                                             |
        // +--------------------------------------------------------------------------------------+
        for (int j = 0; j < mValues.length; j++) {
            System.arraycopy(b4, 0, buf, i, 4);
            i += 4;
        }
        // +<<------------------------------------------------------------------------------------+
        // | Set integer data number.                                                             |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(mValues.length), 0, buf, i, 4);
        i += 4;
        // +<<------------------------------------------------------------------------------------+
        // | Set integer data.                                                                    |
        // +--------------------------------------------------------------------------------------+
        for (int j = 0; j < mValues.length; j++) {
            System.arraycopy(Utils.intToByteArray(mValues[j]), 0, buf, i, 4);
            i += 4;
        }

        return buf;
    }

    @Override
    public boolean fromBytes(byte[] bytes) {
        final int lens[] = checkBytes(bytes);

        if (lens == null) return false;
        byte buf[];
        int i = 16 + lens.length * 4;
        int j = 0;

        buf = new byte[lens[j]];

        // +<<------------------------------------------------------------------------------------+
        // | Get integer data number.                                                             |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(bytes, i, buf, 0, lens[j]);
        final int length = Utils.byteArrayToInt(buf);
        i += lens[j];
        j ++;

        // +<<------------------------------------------------------------------------------------+
        // | Get integer data.                                                                    |
        // +--------------------------------------------------------------------------------------+
        mValues = new int[length];
        for (int k = 0; k < length; k++) {
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            mValues[k] = Utils.byteArrayToInt(buf);
            i += lens[j];
            j ++;
        }
        return true;
    }

    /**
     * It gets the integer value.
     * @param index It is the index of the integer.
     * @return It returns the integer value.
     */
    public int getInteger(int index) {
        return mValues[index];
    }
}
