package com.oozic.net;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;

/**
 * It is the data that contains integers and strings.
 * @author CHEN JIAN WEN
 * [Not Tested]
 */
public class NetDataIntegersStrings extends NetData {
    /**
     * It is the integer value set.
     */
    protected int[] mInts;

    /**
     * It is the string value set.
     */
    protected String[] mStrings;

    /**
     * It constructs the data.
     */
    public NetDataIntegersStrings(int id) {
        this(id, null, null);
    }

    /**
     * It constructs the data.
     * @volume It is the music volume
     */
    public NetDataIntegersStrings(int id, int[] ints, String[] strings) {
        mID = id;

        if (ints == null) {
            mInts = new int[0];
        } else {
            mInts = ints;
        }

        if (strings == null) {
            mStrings = new String[0];
        } else {
            mStrings = strings;
        }
    }

    @Override
    public byte[] toBytes() {
        final ArrayList<byte[]> bufferArray = new ArrayList<byte[]>();

        int length = FIXED_HEAD_SIZE;

        bufferArray.add(Utils.intToByteArray(mInts.length));
        length += 8;

        bufferArray.add(Utils.intToByteArray(mStrings.length));
        length += 8;

        for(int p : mInts) {
            bufferArray.add(Utils.intToByteArray(p));
            length += 8;
        }

        for(String p : mStrings) {
            byte buf[];
            try {
                buf = p.getBytes(STRING_CODING);
            } catch (UnsupportedEncodingException e) {
                buf = p.getBytes();
            } catch (NullPointerException e) {
                buf = new byte[0];
            }
            bufferArray.add(buf);
            length += 4 + buf.length;
        }

        final byte mainBuffer[] = new byte[length];
        int i = FIXED_HEAD_SIZE;
        writeHead(mainBuffer, length, bufferArray.size());

        // Set the data size filed
        for (byte[] p : bufferArray) {
            System.arraycopy(Utils.intToByteArray(p.length), 0, mainBuffer, i, 4);
            i += 4;
        }

        // Set the data field
        for (byte[] p : bufferArray) {
            System.arraycopy(p, 0, mainBuffer, i, p.length);
            i += p.length;
        }

        return mainBuffer;
    }

    @Override
    public boolean fromBytes(byte[] bytes) {
        final int lens[] = checkBytes(bytes);

        if (lens == null) return false;
        byte buf[];
        int i = FIXED_HEAD_SIZE + lens.length * 4;
        int j = 0;

        // Get integer data amount.
        buf = new byte[lens[j]];
        System.arraycopy(bytes, i, buf, 0, lens[j]);
        final int li = Utils.byteArrayToInt(buf);
        if (li < 0) return false;
        i += lens[j];
        j ++;

        // Get string data amount.
        buf = new byte[lens[j]];
        System.arraycopy(bytes, i, buf, 0, lens[j]);
        final int ls = Utils.byteArrayToInt(buf);
        if (ls < 0) return false;
        i += lens[j];
        j ++;

        // Get integer data.
        mInts = new int[li];
        for (int k = 0; k < li; k++) {
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            mInts[k] = Utils.byteArrayToInt(buf);
            i += lens[j];
            j ++;
        }

        // Get string data.
        mStrings = new String[ls];
        for (int k = 0; k < ls; k++) {
            if (lens[j] <= 0) {
                mStrings[k] = null;
                j++;
                continue;
            }
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            try {
                mStrings[k] = new String(buf, STRING_CODING);
            } catch (UnsupportedEncodingException e) {
                mStrings[k] = new String(buf);
            }
            i += lens[j];
            j ++;
        }
        return true;
    }

    /**
     * It gets the integer value.
     * @param index It is the index of the integer.
     * @return It returns the integer value.
     */
    public int getInteger(int index) {
        return mInts[index];
    }

    /**
     * It gets the string.
     * @param index It is the index of the string.
     * @return It returns the string.
     */
    public String getString(int index) {
        return mStrings[index];
    }
}
