package com.oozic.net;

import java.io.UnsupportedEncodingException;

import com.oozic.net.NetData;
import com.oozic.net.Utils;

/**
 * It is the data that contains with simple string.
 * @author CHEN JIAN WEN
 *
 */
public class NetDataString extends NetData {
    /**
     * It is the field number in this class.
     */
    private static final int ITEM_NUMBER = 1;

    /**
     * It is the string.
     */
    protected String mString;

    /**
     * It constructs the data.
     * @param id It is the protocol id.
     */
    public NetDataString(int id) {
        this(id, null);
    }

    /**
     * It constructs the data.
     * @param id It is the protocol id.
     * @param string It is the string to set.
     */
    public NetDataString(int id, String string) {
        mID = id;
        if (string == null) {
            mString = new String();
        } else {
            mString = new String(string);
        }
    }

    @Override
    public byte[] toBytes() {
        byte sbuf[];

        try {
            sbuf = mString.getBytes(STRING_CODING);
        } catch (UnsupportedEncodingException e) {
            sbuf = mString.getBytes();
        }

        final int length =
            4 + 4 + 4 + 4 +
            4 + sbuf.length;

        byte ret[] = new byte[length];
        int i = 0;

        // Set ID.
        System.arraycopy(
                Utils.intToByteArray(mID),
                0, ret, i, 4);
        i += 4;

        // Set Key
        System.arraycopy(Utils.intToByteArray(mKey), 0, ret, i, 4);
        i += 4;

        // Set data length
        System.arraycopy(Utils.intToByteArray(length), 0, ret, i, 4);
        i += 4;

        // Set data field number.
        System.arraycopy(Utils.intToByteArray(ITEM_NUMBER), 0, ret, i, 4);
        i += 4;

        // Set data size field
        System.arraycopy(Utils.intToByteArray(sbuf.length), 0, ret, i, 4);
        i += 4;

        // Set data field,
        System.arraycopy(sbuf, 0, ret, i, sbuf.length);
        i += sbuf.length;

        return ret;
    }

    @Override
    public boolean fromBytes(byte[] bytes) {
        final int lens[] = checkBytes(bytes);

        if (lens == null) return false;
        byte buf[];
        int i = 16 + lens.length * 4;
        int j = 0;

        buf = new byte[lens[j]];
        System.arraycopy(bytes, i, buf, 0, lens[j]);
        try {
            mString = new String(buf, STRING_CODING);
        } catch (UnsupportedEncodingException e) {
            mString = new String(buf);
        }
        i += lens[j];
        j ++;

        return true;
    }

    /**
     * It gets the string stores in the pack.
     * @return It returns the string stores in the pack.
     */
    @Override
    public String toString() {
        return mString;
    }
}
