package com.oozic.net;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;


/**
 * It is the net data with strings..
 * @author CHEN JIAN WEN
 *
 */
public class NetDataStrings extends NetData {
    /**
     * It is the field number in this class.
     */
    private int mItemNumber;

    /**
     * It is the strings.
     */
    private String mStrings[];

    /**
     * It constructs an empty application information.
     * @param id It is the protocol id.
     * @param spy It is the bitmap spy manager.
     */
    protected NetDataStrings(int id) {
        this(id, null);
    }

    /**
     * It constructs the application information with a resolve information.
     * @param id It is the protocol id.
     * @param strings It is the string set.
     */
    protected NetDataStrings(int id, String[] strings) {
        if (strings == null) mStrings = new String[0];
        else mStrings = strings;

        mItemNumber = mStrings.length + 1;

        mID = id;
    }

    /**
     * It sets the strings.
     * @param strings It is the strings to set.
     */
    public void setStrings(String ...strings) {
        mStrings = strings;
        mItemNumber = strings.length + 1;
    }

    @Override
    public byte[] toBytes() {
        final ArrayList<byte[]> bufferArray = new ArrayList<byte[]>();

        int length = 16;

        bufferArray.add(Utils.intToByteArray(mStrings.length));
        length += 8;

        for(int i = 0; i < mStrings.length; i++) {
            byte buffer[];
            try {
                buffer = mStrings[i].getBytes(STRING_CODING);
            } catch (UnsupportedEncodingException e) {
                buffer = mStrings[i].getBytes();
            } catch (NullPointerException e) {
                buffer = new byte[0];
            }
            bufferArray.add(buffer);
            length += 4 + buffer.length;
        }

        byte ret[] = new byte[length];
        int i = 0;

        // Set ID.
        System.arraycopy(Utils.intToByteArray(mID), 0, ret, i, 4);
        i += 4;

        // Set Key
        System.arraycopy(Utils.intToByteArray(mKey), 0, ret, i, 4);
        i += 4;

        // Set the data length.
        System.arraycopy(Utils.intToByteArray(length), 0, ret, i, 4);
        i += 4;

        // Set the item number.
        System.arraycopy(Utils.intToByteArray(mItemNumber), 0, ret, i, 4);
        i += 4;

        // Set the data size filed
        for (byte[] p : bufferArray) {
            System.arraycopy(Utils.intToByteArray(p.length), 0, ret, i, 4);
            i += 4;
        }

        // Set the data field
        for (byte[] p : bufferArray) {
            System.arraycopy(p, 0, ret, i, p.length);
            i += p.length;
        }
        return ret;
    }

    @Override
    public boolean fromBytes(byte[] bytes) {
        final int lens[] = checkBytes(bytes);

        if (lens == null) return false;
        byte buf[];
        int i = 16 + lens.length * 4;
        int j = 0, j1, n;

        // Get number of string.
        buf = new byte[lens[j]];
        System.arraycopy(bytes, i, buf, 0, lens[j]);
        n = Utils.byteArrayToInt(buf);
        mStrings = new String[n];
        i += lens[j];
        j ++;
        n += j;

        // Get strings.
        j1 = j;
        for (; j < n ; j++) {
            if (lens[j] <= 0) {
                mStrings[j - j1] = null;
                continue;
            }
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            try {
                mStrings[j - j1] = new String(buf, STRING_CODING);
            } catch (UnsupportedEncodingException e) {
                mStrings[j - j1] = new String(buf);
            }
            i += lens[j];
        }

        return true;
    }

    /**
     * It gets the specified string.
     * @param index It is the index of string.
     * @return It gets the specified string.
     */
    protected String getString(int index) {
        return mStrings[index];
    }
}
