package com.oozic.net;

import android.os.Looper;
import android.os.SystemClock;
import android.util.Log;

/**
 * It is the thread to process the received net data. 
 * @author CHEN JIAN WEN
 *
 */
class NetDataThread extends Thread {
    /**
     * It is the class tag.
     */
    private static final String TAG = "NetDataThread";

    /**
     * It is the debug switch.
     */
    static final boolean DEBUG = NetManager.DEBUG;

    /**
     * It logs information.
     * @param text It is the text to log.
     */
    final void log_i(String text) {
        if (DEBUG) Log.i(TAG, text);
    }

    /**
     * It is the handler of net data thread.
     */
    NetDataThreadHandler mHandler = null;

    /**
     * It is the net data receiver manager.
     */
    private NetDataReceiverManager mNDRM;

    /**
     * It constructs the net data thread.
     * @param ndrm It is the net data receiver manager.
     */
    public NetDataThread(NetDataReceiverManager ndrm) {
        super();
        setName(TAG + getId());
        mNDRM = ndrm;
    }

    @Override
    public void run() {
        Looper.prepare();

        mHandler = new NetDataThreadHandler(mNDRM);
        log_i(TAG + " is created!");
        Looper.loop();
    }

    /**
     * It makes the thread quit.
     */
    public void quit() {
        boolean loop = false;
        long ts = SystemClock.uptimeMillis();
        do {
            if (mHandler != null) {
                mHandler.sendMessage(mHandler.obtainQuitMessage());
            } else if (isAlive()) {
                loop = true;
                try {
                    Thread.sleep(50);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                    break;
                }
                if (Math.abs(SystemClock.uptimeMillis() - ts) > 200) {
                    break;
                }
            }
        } while(loop);
        log_i("Quit " + TAG + " !");
    }

    /**
     * It process the net data in data thread.  
     * @param pack It is the package to process.
     * @param nu It is the net user who sends the package.
     */
    public void processData(ByteBufferPackage pack, NetUser nu) {
        if (mHandler != null) {
            mHandler.sendMessage(mHandler.obtainDataMessage(pack, nu));
        } else {
            log_i("Thread has not prepared!");
        }
    }
}
