package com.oozic.net;

import android.os.Handler;
import android.os.Looper;
import android.os.Message;

/**
 * It is the handler to process the message in net data thread.
 * @author CHEN JIAN WEN
 *
 */
class NetDataThreadHandler extends Handler {

    /**
     * It is the quit message.
     */
    static final int MSG_QUIT = -1;

    /**
     * It is the process data message.
     */
    static final int MSG_DATA = 0;

    /**
     * It is the net data receiver manager.
     */
    private NetDataReceiverManager mNDRM;

    /**
     * It constructs this handler.
     * @param ndrm It is the net data receiver manager.
     */
    public NetDataThreadHandler(NetDataReceiverManager ndrm) {
        super();
        mNDRM = ndrm;
    }

    @Override
    public void handleMessage(Message msg) {
        final int what = msg.what;
        
        switch (what) {
        case MSG_DATA:
            final Data data = (Data)msg.obj;
            mNDRM.runIfMatch(data.mPackage, data.mUser);
            break;
        case MSG_QUIT:
            Looper.myLooper().quit();
            break;
        }
        
    }

    /**
     * It is the class contains the data to process.
     * @author CHEN JIAN WEN
     *
     */
    private class Data {
        ByteBufferPackage mPackage;
        NetUser mUser;
    }


    /**
     * It obtains the data message to process.
     * @param pack It is the package to process.
     * @param nu It is the user who send the package.
     * @return It returns the message contains the data to processs.
     */
    public Message obtainDataMessage(ByteBufferPackage pack, NetUser nu) {
        final Data data = new Data();
        data.mPackage = pack;
        data.mUser = nu;
        return obtainMessage(MSG_DATA, data);
    }

    /**
     * It obtains the quit message.
     * @return It returns the quit message.
     */
    public Message obtainQuitMessage() {
        return obtainMessage(MSG_QUIT);
    }
}
