package com.oozic.net;

import android.os.Handler;
import android.os.Message;


import com.oozic.library.sdsp.SdspClient;

public class NetHandler extends Handler {
    static final int MSG_CLIENT_CONNECTED = 0x101;
    static final int MSG_CLIENT_UPDATE = 0x102;
    static final int MSG_CLIENT_DISCONNECTED = 0x103;

    private String mIDName;
    private NetUserListener mNSL = null;
    private NetUserManager mNUMgr = null;
    private String mDoNotSeeNames[];

    public NetHandler(NetUserManager nuMgr, NetUserListener listener, String idName, String[] doNotSeeNames) {
        super();

        mNSL = listener;
        mIDName = idName;
        mNUMgr = nuMgr;
        mDoNotSeeNames = doNotSeeNames;
    }

    @Override
    public void handleMessage(Message msg) {
        switch (msg.what) {
        case MSG_CLIENT_CONNECTED: {
            SdspClient client = (SdspClient)msg.obj;
            handleNewClient(client);
            break;
        }

        case MSG_CLIENT_UPDATE:{
            SdspClient client = (SdspClient)msg.obj;
            handleUpdateClient(client);
            break;
        }

        case MSG_CLIENT_DISCONNECTED:
            int native_client = msg.arg2;
            handleRemoveClient(native_client);
            break;
        }
    }

    private void handleNewClient(SdspClient client) {
        NetManager.log_i("New client connected: [" + client.getName() + "]");
        
        if (mNUMgr != null
                && ((mIDName == null) || (client.getName().startsWith(mIDName)))) {
            final int n = mDoNotSeeNames.length;
            for (int i = 0; i < n; i++) {
                if (mDoNotSeeNames[i].equals(client.getName())) {
                    NetManager.log_i("User [" + client.getName() + "] is denided! ");
                    NetManager.log_i("DoNotSee:" + mDoNotSeeNames[i]);
                    return;
                }
            }

            final NetUser user = new NetUser(client);

            final int ret = mNUMgr.add(user);
            NetManager.log_i("User [" + client.getName() + "] is accepted! Mode:" + ret);
            
            if(mNSL != null){
                mNSL.onConnect(user, ret);
            }

        } else {
            NetManager.log_i("User [" + client.getName() + "] is denided! ");
            if (mIDName != null) {
                NetManager.log_i("IDName:" + mIDName);
            }
        }
    }

    private void handleUpdateClient(SdspClient client) {
        if (mNUMgr != null) {
            final NetUser user = mNUMgr.update(client);

            if( (mNSL != null) && (user != null)){
                mNSL.onUpdate(user);
            }
        }
    }

    private void handleRemoveClient(int key) {
        NetManager.log_i("Client disconnected!");
        if (mNUMgr != null) {
            final NetUser user = mNUMgr.remove(key, mNSL);
            if (user != null) {
                NetManager.log_i("Client disconnected: [" + user.getName() + "]");
            } else {
                NetManager.log_e("Client is null. Its key is " + key);
            }
            if( (mNSL != null) && (user != null)){
                mNSL.onDisconnect(user);
            }
        }
    }
}
