package com.oozic.net;

//import android.os.SystemClock;
import android.util.Log;

import com.oozic.library.sdsp.SdspClient;
import com.oozic.library.sdsp.SdspPacketInterface;


/**
 * It is the net packet call back.
 * <BR>
 * It is the internal class of oozic net.
 * @author CHEN JIAN WEN
 * @version 1
 */
class NetPackageCallback extends SdspPacketInterface {

    /**
     * It is the log tag of the class.
     */
    static String TAG = "NetPackageCallback";

    /**
     * It is the net user mananger.
     */
    protected NetUserManager mNUMgr;

    /**
     * It is the net package process trigger mananger.
     */
    protected NetDataReceiverManager mNPRTMgr;

    /**
     * It is the net package listener.
     */
    protected NetPackageListener mNPL;

    /**
     * It constructs the net package callback.
     * @param num It is the net user mananger.
     * <BR>
     * Its value should not be null.
     * @param npptm It is the net package receive trigger manager.
     * <BR>
     * Its value should not be null. 
     * @param npl It is the export listener of receiving net packages.
     */
    NetPackageCallback(
            NetUserManager num,
            NetDataReceiverManager npptm,
            NetPackageListener npl) {
        super();
        mNUMgr = num;
        mNPRTMgr = npptm;

        // Ensure the net package listener is not null, thus it can make the receive process
        // faster.
        if (npl == null) {
            mNPL = new NetPackageListener() {
                @Override
                public void onReceive(NetUser nu, DataPackage dp) {
                    // It do nothing.
                }
            };
        } else {
            mNPL = npl;
        }
    }

    /**
     * onReceive is runned in jni thread. It is called when there is a package is received.
     */
    @Override
    public void onReceive(SdspClient client, byte[] buffer) {
//        mHandler.post(new NetPacketRun(client, buffer));
//        final long ts = SystemClock.uptimeMillis();

        final DataPackage dp = new DataPackage(); 
        
        final NetUser nu = mNUMgr.get(client);
        if (nu == null) return;
        final int type = dp.readPkgType(buffer);
        switch (type) {
        case DataPackage.PKG_TYPE_MOTION_EVENT: {
            final MotionEventPackage p = new MotionEventPackage();
            // +<<----------------------------------------------------------------------------+
            // | Don't read the head of this event, because the head of this type will not be | CHEN JIAN WEN
            // | used any more.                                                               | Optimised in 2012/11/20
            // +------------------------------------------------------------------------------+
            //p.readPkgHeader(mBuffer);
            p.mAction = type;
            p.read(buffer);
            mNPL.onReceive(nu, p);
            break;
        }
        case DataPackage.PKG_TYPE_BYTE_BUFFER: {
            final ByteBufferPackage p = new ByteBufferPackage();
            p.readPkgHeader(buffer);
            p.read(buffer);
            mNPL.onReceive(nu, p);
            mNPRTMgr.requestToProcess(p, nu);
            break;
        }
        case DataPackage.PKG_TYPE_STRING: {
            final StringPackage p = new StringPackage();
            p.readPkgHeader(buffer);
            p.read(buffer);
            mNPL.onReceive(nu, p);
            break;
        }
        case DataPackage.PKG_TYPE_KEY_EVENT: {
            final KeyEventPackage p = new KeyEventPackage();
            // +<<----------------------------------------------------------------------------+
            // | Don't read the head of this event, because the head of this type will not be | CHEN JIAN WEN
            // | used any more.                                                               | Optimised in 2012/11/20
            // +------------------------------------------------------------------------------+
            //p.readPkgHeader(mBuffer);
            p.mAction = type;
            p.read(buffer);
            mNPL.onReceive(nu, p);
            break;
        }
        case DataPackage.PKG_TYPE_FULL_MOTION_EVENT: {
            final FullMotionEventPackage p = new FullMotionEventPackage();
            // +<<----------------------------------------------------------------------------+
            // | Don't read the head of this event, because the head of this type will not be | CHEN JIAN WEN
            // | used any more.                                                               | Optimised in 2012/11/20
            // +------------------------------------------------------------------------------+
            //p.readPkgHeader(mBuffer);
            p.mAction = type;
            p.read(buffer);
            mNPL.onReceive(nu, p);
            break;
        }
        case DataPackage.PKG_TYPE_SENSOR_EVENT: {
            final SensorEventPackage p = new SensorEventPackage();
            // +<<----------------------------------------------------------------------------+
            // | Don't read the head of this event, because the head of this type will not be | CHEN JIAN WEN
            // | used any more.                                                               | Optimised in 2012/11/20
            // +------------------------------------------------------------------------------+
            //p.readPkgHeader(mBuffer);
            p.mAction = type;
            p.read(buffer);
            mNPL.onReceive(nu, p);
            break;
        }
        default: {
            Log.i(TAG, "Package is not supported by sdsp protocol! Ask Jian Wen.");
            break;
        }
        }
    }
}
