package com.oozic.net;

import com.oozic.library.sdsp.SdspClient;
import com.oozic.library.sdsp.SocketAddress;

/**
 * It contains the information of net interface.
 * @author CHEN JIAN WEN
 *
 */
public class NetUser {
    /**
     * It is the name of user.
     */
    public String mName;

    /**
     * It is the user account.
     */
    public String mAccount = null;

    /**
     * It is the image path.
     */
    public String mImagePath = null;

    /**
     * It is the socket address of the user
     */
    private SocketAddress mSocketAddress;

    /**
     * It is the client data.
     */
    private SdspClient mClient;

    /**
     * It is the selected flag.
     */
    public boolean mSelected = false;

    /**
     * It is the log out flag. 
     */
    public boolean mLogOut = false;
    
    public int mKey;

    public NetUser(SdspClient client) {
        mClient = client;
        mName = client.getName();
        mSocketAddress = client.getAddress();
        mKey = client.getNative();
    }
    
    public int getKey(){
        return mKey;
    }

    /**
     * Update Client, name & Key only.
     * @param nu
     */
    public void updateNetUser(NetUser nu) {
        mClient = nu.mClient;
        mName = nu.mName;
        //mSocketAddress = nu.mSocketAddress;
        mKey = nu.mKey;
    }
    
    
    public void setClient(NetUser nu){
        mClient = nu.mClient;
    }
    
    public SdspClient getClient() {
        return mClient;
    }
    
    public String getName() {
        return mName;
    }
    
    public String getIpAddress() {
        if (mSocketAddress != null) {
            return mSocketAddress.getIpAddress();
        }
        return null;
    }
    
    public int getSocketPort() {
        if (mSocketAddress != null) {
            return mSocketAddress.getPort();
        }
        return 0;
    }
    
    /**
     * Update Client, name & Key only.
     * @param client
     */
    public void updateClient(SdspClient client) {
        mClient = client;
        mName = client.getName();
        //mSocketAddress = client.getAddress();
        mKey = client.getNative();
    }
    
    public boolean equals(String ip, int port) {
        if (ip == null) {
            return false;
        }

        int my_port = mSocketAddress.getPort();
        if (my_port != port) {
            return false;
        }

        String my_ip_string = mSocketAddress.getIpAddress();

        if (ip.compareTo(my_ip_string) == 0) {
            return true;
        }

        return false;
    }
    
    public boolean equals(SocketAddress address) {
        if (address == null) {
            return false;
        }

        return mSocketAddress.equals(address);
    }
    
    public boolean equals(SdspClient client) {
        if (client == null) {
            return false;
        }

        SocketAddress address = client.getAddress();

        if (address == null) {
            return false;
        }

        return mSocketAddress.equals(address);
    }
    
    public boolean equals(NetUser nu) {
        if (nu == null)
            return false;

        return mSocketAddress.equals(nu.mSocketAddress);
    }
    
    @Override
    public boolean equals(Object nu) {
        if (nu == null) return false;
        
        if (!(nu instanceof NetUser)) return false;
        
        NetUser user = (NetUser)nu;
        
        return mSocketAddress.equals(user.mSocketAddress);
    }
}