package com.oozic.net;

/**
 * It is the sensor event package.
 * @author CHEN JIAN WEN
 *
 */
public class SensorEventPackage extends DataPackage {

    /**
     * It is the sensor event type. Please refer the sensor type declared
     * in android.hardware.SensorManager.
     */
    int mSensorType;

    /**
     * It is the data in sensor.
     */
    float mSensorData[];

    /**
     * It constructs the key event package.
     */
    public SensorEventPackage() {
        mType = PKG_TYPE_SENSOR_EVENT;
    }

    /**
     * It sets the sensor event
     * @param type It is the sensor type.
     * @param data It is the sensor data
     */
    public void setSensorEvent(int type, float ... data) {
        mSensorType = type;
        mSensorData = data;
    }

    /**
     * It converts the data to buffer.
     * @return It returns the byte array buffer with data
     */
    private byte[] dataToBuffer() {
        int dataSize = 0;
        if (mSensorData == null) dataSize = 8;
        else dataSize = 8 + mSensorData.length * 4;

        final byte[] data = new byte[dataSize];

        int i = 0;
        System.arraycopy(Utils.intToByteArray(mSensorType), 0, data, i, 4);
        i += 4;

        System.arraycopy(Utils.intToByteArray(mSensorData.length), 0, data, i, 4);
        i += 4;

        for (int j = 0; j < mSensorData.length; j++) {
            System.arraycopy(Utils.floatToByteArray(mSensorData[j]), 0, data, i, 4);
            i += 4;
        }

        return data;
    }

    /**
     * It converts the buffer to value.
     * @param buffer It is the buffer to convert.
     */
    private void bufferToData(byte[] buffer) {
        byte buf[] = new byte[4];
        int i = 0;
        System.arraycopy(buffer, i, buf, 0, 4);
        mSensorType = Utils.byteArrayToInt(buf);
        i += 4;

        System.arraycopy(buffer, i, buf, 0, 4);
        final int length = Utils.byteArrayToInt(buf);
        i += 4;

        mSensorData = new float[length];
        for (int j = 0; j < length; j++ ) {
            System.arraycopy(buffer, i, buf, 0, 4);
            mSensorData[j] = Utils.byteArrayToFloat(buf);
            i += 4;
        }
    }

    /**
     * It reads the data from source.
     * @param in It is data source.
     */
    public void read(byte[] in) {
        final byte blen[] = new byte[4];
        int i = DataPackage.PKG_HEADER_SIZE;
        System.arraycopy(in, i, blen, 0, 4);
        i += 4;
        final int len = Utils.byteArrayToInt(blen);
        if (len > 0) {
            final byte b[] = new byte[len];
            System.arraycopy(in, i, b, 0, len);
            bufferToData(b);
        }
    }

    /**
     * It write the data to target buffer.
     * @return It returns the data buffer to write.
     */
    public byte[] write() {
        final byte buffer[] = dataToBuffer();
        final int len = buffer.length;
        final byte blen[] = Utils.intToByteArray(len);
        int i = DataPackage.PKG_HEADER_SIZE;
        final byte out[] = new byte[len + 4 + i];
        System.arraycopy(blen, 0, out, i, 4);
        i += 4;
        System.arraycopy(buffer, 0, out, i, len);
        return out;
    }

    public int getSensorType() {
        return mSensorType;
    }

    public float[] getSensorData() {
        return mSensorData;
    }
}