package com.oozic.net;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;


public class StringPackage extends DataPackage {
    String mContent;

    public StringPackage() {
        mContent = null;
        mType = PKG_TYPE_STRING;
    }

    public StringPackage(String content) {
        mContent = content;
        mType = PKG_TYPE_STRING;
    }

    @Override
    public void setString(String string) {
        mContent = string;
    }

    @Override
    public String getString() {
        return mContent;
    }

    @Override
    public byte[] getByteBuffer() {
        return mContent.getBytes();
    }

    protected void write(DataOutputStream dos) throws IOException {
        writePkgHeader(dos);

        if (mContent == null || mContent.length() == 0) {
            dos.writeInt(0);
        } else {
            dos.writeInt(mContent.length());
            dos.writeUTF(mContent);
        }
    }

    protected void read(DataInputStream dis) throws IOException {
        readPkgHeader(dis);

        int length = dis.readInt();
        if (length == 0) {
            mContent = null;
        } else {
            mContent = dis.readUTF();
        }
    }

    /**
     * It reads the data from source.
     * @param in It is data source.
     */
    public void read(byte[] in) {
        final byte blen[] = new byte[4];
        int i = DataPackage.PKG_HEADER_SIZE;
        System.arraycopy(in, i, blen, 0, 4);
        i += 4;
        final int len = Utils.byteArrayToInt(blen);
        if (len > 0) {
            final byte bstring[] = new byte[len];
            System.arraycopy(in, i, bstring, 0, len);
            try {
                mContent = new String(bstring, "utf-8");
            } catch (UnsupportedEncodingException e) {
                mContent = new String(bstring);
            }
        }
    }

    /**
     * It write the data to target buffer.
     * @return It returns the data buffer to write.
     */
    public byte[] write() {
        byte bytes[];
        try {
            bytes = mContent.getBytes("utf-8");
        } catch (UnsupportedEncodingException e) {
            bytes = mContent.getBytes();
        }
        final int len = bytes.length;
        final byte blen[] = Utils.intToByteArray(len);
        int i = DataPackage.PKG_HEADER_SIZE;
        final byte out[] = new byte[len + 4 + i];
        if (len > 0) {
            System.arraycopy(blen, 0, out, i, 4);
            i += 4;
            System.arraycopy(bytes, 0, out, i, len);
            return out;
        }
        return null;
    }
}
