package com.oozic.net;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.Enumeration;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

import android.util.Log;

/**
 * A utility toolkit for the related application.
 * 
 * @author LIANG QI LIANG, LIU GUO PING & CHEN JIAN WEN
 * 
 */
public class Utils {

    private static final String[] DEFAULT_HOSTNAME = { "uap0", "wlan0", "tiwlan0", "wifi0", "eth0", };
    private static final String AES_PWD_PREFIX = "#!pWD_ooShare_";
    private static final int AES_KEY_LENGTH = 256;

    /**
     * It converts a short value to a byte array.
     * @param value It is the short value to covert.
     * @return It returns a 2 bytes long byte array.
     */
    public static final byte[] shortToByteArray(short value) {
        return new byte[] { (byte)(value >>> 8), (byte) value};
    }

    /**
     * It converts a float value to a byte array.
     * @param value It is the float value to covert.
     * @return It returns a 4 bytes long byte array.
     */
    public static final byte[] floatToByteArray(float value) {
        final int valueF = Float.floatToRawIntBits(value);
        return intToByteArray(valueF);
    }

    /**
     * It converts a integer value to a byte array.
     * @param value It is the integer value to covert.
     * @return It returns a 4 bytes long byte array.
     */
    public static final byte[] intToByteArray(int value) {
        return new byte[] { (byte) (value >>> 24), (byte) (value >>> 16), (byte) (value >>> 8),
                (byte) value };
    }

    /**
     * It converts a long value to a byte array.
     * @param value It is the integer value to covert.
     * @return It returns a 8 bytes long byte array.
     */
    public static final byte[] longToByteArray(long value) {
        return new byte[] { (byte) (value >>> 56), (byte) (value >>> 48), (byte) (value >>> 40),
                (byte) (value >>> 32), (byte) (value >>> 24), (byte) (value >>> 16),
                (byte) (value >>> 8), (byte) value };
    }

    /**
     * It converts a byte array to short value.<BR>
     * <B>Note</B>, the byte array must not be null and its length should not be shorter than 2.
     * @param b It the byte array to convert.
     * @return It returns a short value.
     */
    public static final short byteArrayToShort(byte[] b) {
        return (short)(((b[0] & 0xff) << 8) | (b[1] & 0xff));
    }

    /**
     * It converts a byte array to float value.<BR>
     * <B>Note</B>, the byte array must not be null and its length should not be shorter than 4.
     * @param b It the byte array to convert.
     * @return It returns a float value.
     */
    public static final float byteArrayToFloat(byte[] b) {
        final int valueInt = byteArrayToInt(b);
        return Float.intBitsToFloat(valueInt);
    }

    /**
     * It converts a byte array to float.<BR>
     * <B>Note</B>, the byte array must not be null and its length should not be shorter than 4.
     * @param b It the byte array to convert.
     * @return It returns a float value.
     */
    public static final int byteArrayToInt(byte[] b) {
        return ((b[0] & 0xff) << 24) | ((b[1] & 0xff) << 16)
                | ((b[2] & 0xff) << 8) | (b[3] & 0xff);
    }

    /**
     * It converts a byte array to long value.<BR>
     * <B>Note</B>, the byte array must not be null and its length should not be shorter than 8.
     * @param b It the byte array to convert.
     * @return It returns a float value.
     */
    public static final long byteArrayTolong(byte[] b) {
        int i1 = ((b[0] & 0xff) << 24) | ((b[1] & 0xff) << 16)
                | ((b[2] & 0xff) << 8) | (b[3] & 0xff);
        int i2 = ((b[4] & 0xff) << 24) | ((b[5] & 0xff) << 16)
                | ((b[6] & 0xff) << 8) | (b[7] & 0xff);

        return ((i1 & 0xffffffffL) << 32) | (i2 & 0xffffffffL);
    }

    public static InetAddress getLocalHost() {
        try {
            
            Enumeration<NetworkInterface> netInterfaces = NetworkInterface.getNetworkInterfaces();

            if (netInterfaces != null) {

                NetworkInterface defaultIf = null;
                while (netInterfaces.hasMoreElements()) {
                    NetworkInterface netif = netInterfaces.nextElement();
                    if (defaultIf == null) {
                        defaultIf = netif;
                    }

                    String name = netif.getName();

                    for (String host : DEFAULT_HOSTNAME) {
                        if (host.equals(name)) {
                            Enumeration<InetAddress> ips = netif.getInetAddresses();
                            while (ips.hasMoreElements()) {
                                InetAddress net = (InetAddress) ips.nextElement();
                                if (net != null) {
                                    return net;
                                }
                            }
                        }
                    }
                }

                if (defaultIf != null) {
//                    Log.w("OOSA3", "No appointed interface found, the first interface is used.");
                    Enumeration<InetAddress> ips = defaultIf.getInetAddresses();
                    while (ips.hasMoreElements()) {
                        InetAddress net = (InetAddress) ips.nextElement();
                        if (net != null) {
                            return net;
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }
    
    public static byte[] getLocalIP() {
        byte[] ip = { 0, 0, 0, 0 };
        try {
            Enumeration<NetworkInterface> netInterfaces = NetworkInterface.getNetworkInterfaces();

            if (netInterfaces != null) {

                NetworkInterface defaultIf = null;
                while (netInterfaces.hasMoreElements()) {
                    NetworkInterface netif = netInterfaces.nextElement();
                    if (defaultIf == null) {
                        defaultIf = netif;
                    }

                    String name = netif.getName();

                    for (String host : DEFAULT_HOSTNAME) {
                        if (host.equals(name)) {
                            Enumeration<InetAddress> ips = netif.getInetAddresses();
                            while (ips.hasMoreElements()) {
                                InetAddress net = (InetAddress) ips.nextElement();
                                ip = net.getAddress();
                                if (ip != null && ip.length == 4) {
                                    return ip;
                                }
                            }
                        }
                    }
                }

                if (defaultIf != null) {
//                    Log.w("OOSA3", "No appointed interface found, the first interface'ip is used.");
                    Enumeration<InetAddress> ips = defaultIf.getInetAddresses();
                    while (ips.hasMoreElements()) {
                        InetAddress net = (InetAddress) ips.nextElement();
                        ip = net.getAddress();
                        if (ip != null && ip.length == 4) {
                            return ip;
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ip;
    }
    
    public static void dumpInterface() {
        try {
            Enumeration<NetworkInterface> netInterfaces = NetworkInterface.getNetworkInterfaces();

            while (netInterfaces.hasMoreElements()) {
                NetworkInterface ni = (NetworkInterface) netInterfaces.nextElement();
                Log.e("OOSA3", "name = " + ni.getName());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public static byte[] getLocalIP(String iface) {
        byte[] ip = { 0, 0, 0, 0 };
        try {
            Enumeration<NetworkInterface> netInterfaces = NetworkInterface.getNetworkInterfaces();

            while (netInterfaces.hasMoreElements()) {

                NetworkInterface ni = (NetworkInterface) netInterfaces.nextElement();
                
                if (iface.equals(ni.getName())) {
                    Enumeration<InetAddress> ips = ni.getInetAddresses();
                    while (ips.hasMoreElements()) {
                        InetAddress net = (InetAddress) ips.nextElement();
                        ip = net.getAddress();
                        if (ip != null && ip.length == 4) {
                            return ip;
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ip;
    }
    
    protected static byte[] encryptData(int serialId, byte[] data) {
        if (serialId < 0 || data == null) return null;
        byte[] encrypted = null;
        try {
            String password = AES_PWD_PREFIX.concat(Integer.toString(serialId));
            KeyGenerator kgen = KeyGenerator.getInstance("AES");
            kgen.init(AES_KEY_LENGTH, new SecureRandom(password.getBytes()));
            SecretKey secretKey = kgen.generateKey();
            byte[] raw = secretKey.getEncoded();
            SecretKeySpec skeySpec = new SecretKeySpec(raw, "AES");
            
            Cipher cipher = Cipher.getInstance("AES");
            cipher.init(Cipher.ENCRYPT_MODE, skeySpec);
            encrypted = cipher.doFinal(data);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (NoSuchPaddingException e) {
            e.printStackTrace();
        } catch (InvalidKeyException e) {
            e.printStackTrace();
        } catch (IllegalBlockSizeException e) {
            e.printStackTrace();
        } catch (BadPaddingException e) {
            e.printStackTrace();
        }
        
        return encrypted;
    }
    
    protected static byte[] decryptData(int serialId, byte[] data) {
        if (serialId < 0 || data == null) return null;
        byte[] decrypted = null;
        try {
            String password = AES_PWD_PREFIX.concat(Integer.toString(serialId));
            KeyGenerator kgen = KeyGenerator.getInstance("AES");
            kgen.init(AES_KEY_LENGTH, new SecureRandom(password.getBytes()));
            SecretKey secretKey = kgen.generateKey();
            byte[] raw = secretKey.getEncoded();
            SecretKeySpec skeySpec = new SecretKeySpec(raw, "AES");
            
            Cipher cipher = Cipher.getInstance("AES");
            cipher.init(Cipher.DECRYPT_MODE, skeySpec);
            decrypted = cipher.doFinal(data);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (NoSuchPaddingException e) {
            e.printStackTrace();
        } catch (InvalidKeyException e) {
            e.printStackTrace();
        } catch (IllegalBlockSizeException e) {
            e.printStackTrace();
        } catch (BadPaddingException e) {
            e.printStackTrace();
        }
        
        return decrypted;
    }
}
