package com.oozic.net.fdt;

import com.oozic.net.NetUser;

import android.os.Handler;
import android.os.Looper;
import android.os.Message;

/**
 * It is the handler to process FDT events.
 * @author CHEN JIAN WEN
 *
 */
class FDTHandler extends Handler {

    /**
     * It is the quit message.
     */
    private static final int MSG_QUIT = -1;

    /**
     * It is the send file message.
     */
    private static final int MSG_SEND_FILE = 0;

    /**
     * It is the file sender manager.
     */
    private FileSenderManager mFSM;

    /**
     * It constructs the FDT handler.
     * @param fdtm It is FDT manager.
     */
    FDTHandler(FDTManager fdtm) {
        super();

        mFSM = new FileSenderManager(fdtm);
    }

    @Override
    public void dispatchMessage(Message msg) {
        final int what = msg.what;

        switch(what) {
        case MSG_SEND_FILE: {
            // Sends the file or make it into the file send task manager.
            if (msg.obj instanceof FileSendTask) {
                sendFile((FileSendTask)msg.obj);
            }
            break;
        }
        case MSG_QUIT: {
            mFSM.destroy();
            // Quit the looper.
            Looper.myLooper().quit();
            break;
        }
        }
    }

    /**
     * It obtains the quit message.
     * @return It returns the quit message.
     */
    Message obtainQuitMessage() {
        return obtainMessage(MSG_QUIT);
    }


    /**
     * It obtains the send file message.
     * @param path It is the file path.
     * @param target It is the target to send to.
     * @param fsl It is the file send listener.
     * @return It returns the send file message.
     */
    Message obtainSendFileMessage(String path, NetUser target, FileSendListener fsl) {
        final FileSendTask fst = new FileSendTask(path, target, fsl);
        return obtainMessage(MSG_SEND_FILE, fst);
    }

    /**
     * It sends the file.
     * @param fst It is the send file task.
     */
    private void sendFile(final FileSendTask fst) {
        FileSender fs = mFSM.createSender();
        if (!fs.send(fst)) {
            mFSM.remove(fs);
        }
    }
}
