package com.oozic.net.fdt;

import com.oozic.net.NetUser;

import android.os.Looper;
import android.os.SystemClock;

/**
 * It is the main thread of FDT.
 * @author CHEN JIAN WEN
 
 */
class FDTThread extends Thread {

    /**
     * It is the FDT handler to process events.
     */
    FDTHandler mHandler;

    /**
     * It is the FDT manager.
     */
    FDTManager mFDTM;

    /**
     * It constructs the FDT thread.
     * @param It is the FDT manager.
     */
    FDTThread(FDTManager fdtm) {
        super();
        mFDTM = fdtm;
    }

    @Override
    public void run() {
        Looper.prepare();
        mHandler = new FDTHandler(mFDTM);
        boolean keep = false;
        do {
            keep = false;
            try {
                Looper.loop();
            } catch (Exception e) {
                e.printStackTrace();
                keep = true;
            }
        } while (keep);
    }

    /**
     * It makes the FDT thread quit the loop and destroy. 
     */
    void quit() {
        boolean keep = false;
        long ts = SystemClock.uptimeMillis();
        do {
            keep = false;
            if (mHandler != null) {
                mHandler.sendMessage(mHandler.obtainQuitMessage());
            } else if (isAlive()) {
                keep = true;
                try {
                    Thread.sleep(50);
                } catch (InterruptedException e) {
                    break;
                }
                if (Math.abs(SystemClock.uptimeMillis() - ts) > 200) {
                    break;
                }
            }
        } while(keep);
    }
    
    /**
     * It sends a file to 
     * @param path It is the file path.
     * @param target It is the target to send file.
     * @param fsl It is the file send listener that calls by FDT in the whoel file send process.
     */
    void sendFile(String path, NetUser target, FileSendListener fsl) {
        if (mHandler != null) {
            mHandler.sendMessage(mHandler.obtainSendFileMessage(path, target, fsl));
        }
    }
}
