package com.oozic.net.fdt;

import android.os.Handler;
import android.os.Looper;

import com.oozic.net.DataPackage;
import com.oozic.net.DataPackageFactory;
import com.oozic.net.NetManager;
import com.oozic.net.NetUser;

/**
 * It is the file accepter to decide if a file should be received.
 * @author CHEN JIAN WEN
 *
 */
class FileAccepter extends Thread {
    /**
     * It is the file direct transfer manager.
     */
    private FDTManager mFDTM;

    /**
     * It is the block size of the receiver.
     */
    private int mBlockSize;

    /**
     * It is the command key.
     */
    private int mCmdKey;

    /**
     * It is the net user.
     */
    private NetUser mNetUser;

    /**
     * It is the file path.
     */
    private String mPath;

    /**
     * It is the file size.
     */
    private long mSize;

    /**
     * It is the count to touch the foreign device.
     */
    private int mTouchCount;

    /**
     * It is the file accept listener.
     */
    private FileAcceptListener mFAL;
    /**
     * 
     * @param fdtm It is FDT manager.
     * @param key It is the key of command.
     * @param nu It is the user who request to send a file.
     * @param path It is the path of the file in foreign client.
     * @param size It is the file size.
     * @param blockSize It is the file block size.
     * @param fal It is the file accept listener.
     */
    FileAccepter(FDTManager fdtm, int key, NetUser nu, String path, long size, int blockSize, FileAcceptListener fal) {
        super();
        mFDTM = fdtm;
        mCmdKey = key;
        mNetUser = nu;
        mPath = path;
        mSize = size;
        mBlockSize = blockSize;
        mTouchCount = 0;
        mFAL = fal;
    }

    @Override
    public void run() {
        if (mFAL == null) return;

        try {
            while(mFAL.isOnHold()) {
                if (mTouchCount == 0) {
                    final NetManager nm = mFDTM.mNetManager;
                    // It just make the receiver hold
                    final Data_SendFileAck ack = new Data_SendFileAck(-1, 0);
                    ack.setKey(mCmdKey);
                    final DataPackage pkg = DataPackageFactory.peekDataPackage(DataPackage.PKG_TYPE_BYTE_BUFFER);
                    pkg.setAction(DataPackage.ACTION_REPLY);
                    pkg.setByteBuffer(ack.toBytes());

                    nm.sendPackage(pkg, mNetUser);
                }
                mTouchCount ++;
                mTouchCount %= 5;
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                    break;
                }
            }
    
            if (mFAL.isAccepted()) {
                // Create a receiver to receive the file.
                final Handler h = new Handler(Looper.getMainLooper());
                h.post(
                    new Runnable() {
                        @Override
                        public void run() {
                            mFAL.onStart(new FileReceiver(mFDTM, mCmdKey, mNetUser, mPath, mSize, mBlockSize));
                        }
                    });
            } else {
                final NetManager nm = mFDTM.mNetManager;
                // It just make the receiver hold
                final Data_SendFileAck ack = new Data_SendFileAck(-2, 0);
                ack.setKey(mCmdKey);
                final DataPackage pkg = DataPackageFactory.peekDataPackage(DataPackage.PKG_TYPE_BYTE_BUFFER);
                pkg.setAction(DataPackage.ACTION_REPLY);
                pkg.setByteBuffer(ack.toBytes());

                nm.sendPackage(pkg, mNetUser);
                return;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }
    }
}
