package com.oozic.net.fdt;

import java.io.File;

import android.os.Build;

import com.oozic.net.ByteBufferPackage;
import com.oozic.net.DataPackage;
import com.oozic.net.DataPackageFactory;
import com.oozic.net.NetData;
import com.oozic.net.NetDataReceiverListener;
import com.oozic.net.NetDataReceiverManager;
import com.oozic.net.NetManager;
import com.oozic.net.NetUser;

/**
 * It is the file receiver manager.
 * @author CHEN JIAN WEN
 *
 */
class FileReceiverManager implements NetDataReceiverListener {

    /**
     * It is the FDT manager.
     */
    FDTManager mFDTM;

    /**
     * It is the file accept listener.
     */
    FileAcceptListener mFAL;

    /**
     * It constructs the file receiver.
     * @param fdtm It is the file direct transceiver manager.
     * @param fal It is the file accept listener.
     */
    FileReceiverManager(FDTManager fdtm, FileAcceptListener fal) {
        mFDTM = fdtm;
        mFAL = fal;

        // Create the file request receiver.
        final NetManager nm = fdtm.mNetManager;
        final NetDataReceiverManager ndrm = nm.getNetDataReceiverManager();

        ndrm.createReceiver(
                DataPackage.ACTION_REQUEST,
                NetData.ID_FILE_TRANSCEIVE_REQ,
                0, this, true);
    }

    /**
     * It destroys the file receiver manager.
     */
    void destroy() {
        final NetManager nm = mFDTM.mNetManager;
        final NetDataReceiverManager ndrm = nm.getNetDataReceiverManager();
        ndrm.kill(
                DataPackage.ACTION_REQUEST,
                NetData.ID_FILE_TRANSCEIVE_REQ,
                0);
    }

    @Override
    public void run(int key, ByteBufferPackage pack, NetUser nu) {
        // Check the package.
        Data_SendFileReq req = new Data_SendFileReq();
        if (!req.fromBytes(pack.getByteBuffer())) return;

        final NetManager nm = mFDTM.mNetManager;

        if (Build.VERSION.SDK_INT >= 9) {
            // Check the local space
            final long size = req.getSize();
            final File dir = new File(mFDTM.mDirectory);
            final long freeSpace = dir.getFreeSpace();
            if (freeSpace < size) {
                // Send remote space error back.
                final Data_SendFileErr err = new Data_SendFileErr(FDTManager.ERROR_CODE_REMOTE_IO);
                err.setKey(key);
                final DataPackage pkg =
                    DataPackageFactory.peekDataPackage(DataPackage.PKG_TYPE_BYTE_BUFFER);
                pkg.setAction(DataPackage.ACTION_REPLY);
                pkg.setByteBuffer(err.toBytes());
                nm.sendPackage(pkg, nu);
                return;
            }
        }

        // Create a receiver to receive the file.
        new FileAccepter(mFDTM, key, nu, req.getPath(), req.getSize(), mFDTM.mBlockSize, mFAL).start();
    }

    @Override
    public void onTimeOut(int action, int id, int key) {
        // It cannot be here, skip this method.
    }
}
