package com.oozic.time;

import android.os.Handler;
import android.util.Log;

/**
 * A monitor will create a thread to see if a user request is matched.
 * @author CHEN JIAN WEN
 *
 */
public class Monitor implements Runnable {
    /**
     * It is the log tag.
     */
    private static final String TAG = "Monitor";

    /**
     * It is the handler of a thread. Generally, it is the UI thread.
     */
    protected Handler mHandler = null;

    /**
     * It is the monitor check period.
     */
    protected int mPeriod;

    /**
     * It is the flag that marks if the monitor is running.
     */
    protected boolean mRunning = false;

    /**
     * It is the flag that let the monitor stop.
     */
    protected boolean mStop = false;

    /**
     * It is the flag that decides the monitor keep run or not after the monitor is triggered.
     */
    protected boolean mLoop;

    /**
     * It is the handler to monitor actions.
     */
    protected MonitorActions mActions = null;

    /**
     * It is the monitor thread.
     */
    protected Thread mThread;

    /**
     * It constructs the monitor.
     * @param period It is the period between each check.
     * @param loop It decides if the monitor is kept after its trigger.
     * @param action It is the monitor actions.
     */
    public Monitor(int period, boolean loop, MonitorActions action) {
        mHandler = new Handler();
        if (period < 200) {
            mPeriod = 200;
        } else  mPeriod = period;
        mLoop = loop;

        if (action != null) {
            mActions = action;
            mRunning = true;
            mThread = new Thread(this);
            mThread.start();
        }
    }

    /**
     * It is the the process in the monitor thread.
     */
    @Override
    public void run() {

        do {
            while(!mStop) {
                final int event = mActions.isTriggered();
                if (event < 0) {
                    try {
                        Thread.sleep(mPeriod);
                    } catch (InterruptedException e) {
                        Log.e(TAG, e.toString());
                    }
                } else {
                    mHandler.post(
                            new Runnable() {
                                @Override
                                public void run() {
                                    mActions.onTriggered(event);
                                }
                            });
                    break;
                }
            }
        }while(!mStop && mLoop);
        mRunning = false;
    }

    /**
     * It stop the monitor. Note, it does not stop the monitor immediately.
     */
    public void stop() {
        mStop = true;
        mThread.interrupt();
    }

    /**
     * It returns the running status of the monitor.
     * @return It returns the running status of the monitor.
     */
    public boolean isRunning() {
        return mRunning;
    }
}
