package com.oozic.time;

import java.util.Timer;
import java.util.TimerTask;

import android.os.Handler;

public class TimeTrigger {
    /**
     * Timer of time trigger
     */
    private Timer mTT;

    /**
     * ANDROID OS handler used to post runnable to main thread.
     */
    Handler mH;

    /**
     * Timer task of time trigger.
     */
    private TimeTriggerTask mTTTask;

    /**
     * Constructs the time trigger.
     */
    public TimeTrigger() {
        // Initialize the variables.
        this(true);
    }

    public TimeTrigger(boolean mainThread) {
        mTT = null;
        mTTTask = null;
        if (mainThread) mH = new Handler();
        else mH = null;
    }
    /**
     * Start a time line.
     * @param begin Begin value of time line.
     * @param end End value of time line.
     * @param duration Duration in millisecond unit.
     * @param fps Frame per second. 
     * @param onActive The runnable runs when time line task is active.
     * @param onFinish The runnable runs when time line task is finished.
     */
    public void start(final int delay, final Runnable onTrigger) {
        int d;

        if (delay < 0) d = 0;
        else d = delay;

        if (mTTTask != null) {
            if (mH != null) mH.removeCallbacks(mTTTask.mOnTrigger);
            mTT.cancel();
            mTTTask.cancel();
        }

        mTT = new Timer();
        mTTTask = new TimeTriggerTask(d, onTrigger);
        mTT.schedule(mTTTask, d);
    }

    /**
     * Clear the time line.
     */
    public void clear() {
        if (mTTTask != null) {
            if (mH != null) mH.removeCallbacks(mTTTask.mOnTrigger);
            mTT.cancel();
            mTTTask.cancel();
            mTT = null;
            mTTTask = null;
        }
    }

    private class TimeTriggerTask extends TimerTask {
        /**
         * Call back function runs when the time trigger is active.
         */
        final protected Runnable mOnTrigger;

        /**
         * Constructor of this class
         * @param delay Delay in millisecond unit.
         * @param onTrigger The runnable runs when time task is triggered.
         */
        public TimeTriggerTask(final int delay, final Runnable onTrigger) {
            mOnTrigger = onTrigger;
        }

        @Override
        public void run() {
            if (mOnTrigger != null) {
                if (mH != null) mH.post(mOnTrigger);
                else mOnTrigger.run();
            }
        }
    }
}
