package com.oozic.view;

import android.content.Context;
import android.graphics.Canvas;
import android.view.View;

/**
 * It is a view that can register a on draw listener, thus, programmer can draw something in view
 * without rewrite a view.
 * @author CHEN JIAN WEN
 *
 */
public class FreeDrawView extends View {

    /**
     * It is the on draw listener.
     */
    protected OnDrawListener mOnDrawListener;

    /**
     * It constructs the free draw view.
     * @param context
     */
    public FreeDrawView(Context context) {
        super(context);
    }

    @Override
    protected void onDraw(Canvas canvas) {
        if (mOnDrawListener != null) mOnDrawListener.onDraw(canvas);
    }
    /**
     * It sets the on draw listener.
     * @param listener It is the listener to set. 
     */
    public void setOnDrawListener(OnDrawListener listener) {
        mOnDrawListener = listener;
    }

    public interface OnDrawListener {
        /**
         * It is called when a free-draw view is drawn.
         * @param canvas It is the canvas to draw.
         */
        public void onDraw(Canvas canvas);
    }
}
